﻿/*
Code written/modified by: Dean Lunz (aka Crated by: X)
Web: http://www.createdbyx.com/ 
NOTICE: The code in this file does not fall under the Zider licence.
  
 Original source for this code comes from the tutorial "Animation Sets" found on the
 http://www.codeanxiety.com/ website.
 Original Code Author: Justin "Kukyona" O'Dell -> http://codeanxiety.com/user/Profile.aspx?UserID=2100
 */
using System;
using Microsoft.Xna.Framework;

namespace Zider.Animation
{
    /// <summary>
    /// Class to store our animation information.
    /// </summary>
    public sealed class AnimationInfo
    {
        // set up variables and their default values
        private int startFrame = 0;
        private int endFrame = 0;
        private float frameRate = 15f;
        private bool loops = false;

        /// <summary>
        /// Default constructor.
        /// </summary>
        /// <param name="startFrame">The index of the first frame in the animation.</param>
        /// <param name="endFrame">The index of the last frame in the animation.</param>
        /// <param name="frameRate">The rate of how many frames are played in one second.</param>
        /// <param name="loops">Whether or not this animation should start over when it finishes.</param>
        public AnimationInfo( int startFrame, int endFrame, float frameRate, bool loops )
        {
            this.startFrame = startFrame;
            this.endFrame = endFrame;
            this.frameRate = frameRate;
            this.loops = loops;
        }

        /// <summary>
        /// Gets the index of the first frame in the animation.
        /// </summary>
        public int StartFrame
        {
            get { return startFrame; }
        }

        /// <summary>
        /// Gets the index of the last frame in the animation.
        /// </summary>
        public int EndFrame
        {
            get { return endFrame; }
        }

        /// <summary>
        /// Gets the rate of how many frames are played in one second.
        /// </summary>
        public float FrameRate
        {
            get { return frameRate; }
        }

        /// <summary>
        /// Gets whether or not this animation should start over when it finishes.
        /// </summary>
        public bool Loops
        {
            get { return loops; }
        }
    }
}
