/*
Created by: X
http://www.createdbyx.com/

The Zider engine is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/
*/

#region Using Statements
using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Nuclex.Fonts;
#endregion

namespace Zider.Classes
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public class GUIWidget : xFripperyXNA.Control, Interfaces.IGUIWidget
    {
        public delegate void BeforeTextureNameEventHandler(out string Name);
        public delegate void AfterTextureNameEventHandler(out string Name);

        private Texture2D tex = null;
        private Rectangle sourceRectangle = Rectangle.Empty;
        private Color color = Color.White;
        private string textureName = string.Empty;
        private bool textureNeedsUpdate = false;
        private BeforeTextureNameEventHandler beforeTexName;
        private AfterTextureNameEventHandler afterTexName;
        private Zider.GetTexture getTex;
        protected ZiderGame game;
        private string text;
        private TextAlignment textAlignment = TextAlignment.HorizontallyCentered | TextAlignment.VerticallyCentered;
        private Nuclex.Fonts.BitmapFont font;
        private Color textColor = Color.White;
        private StringCollection script = new StringCollection();


        public StringCollection Script
        {
            get { return script; }
            set { script = value; }
        }   	

        public Color TextColor
        {
            get { return textColor; }
            set { textColor = value; }
        }

        public Nuclex.Fonts.BitmapFont Font
        {
            get { return font; }
            set { font = value; }
        }

        public Nuclex.Fonts.TextAlignment TextAlignment
        {
            get { return textAlignment; }
            set { textAlignment = value; }
        }

        public string Text
        {
            get { return text; }
            set { text = value; }
        }

        public ZiderGame Game
        {
            get { return game; }
        }

        public Zider.GetTexture GetTexture
        {
            get { return getTex; }
            set { getTex = value; }
        }

        public event BeforeTextureNameEventHandler BeforeTextureName
        {
            add { this.beforeTexName += value; }
            remove { this.beforeTexName -= value; }
        }

        public event AfterTextureNameEventHandler AfterTextureName
        {
            add { this.afterTexName += value; }
            remove { this.afterTexName -= value; }
        }

        public virtual void Initilize(ZiderGame game)
        {
            if (game == null)
            {
                throw new ArgumentNullException("game");
            }
            this.game = game;
        }

        public Color Color
        {
            get { return color; }
            set { color = value; }
        }

        public Rectangle SourceRectangle
        {
            get { return sourceRectangle; }
            set { sourceRectangle = value; }
        }


        public int SourceLeft
        {
            get { return sourceRectangle.X; }
            set { sourceRectangle.X = value; }
        }

        public int SourceTop
        {
            get { return sourceRectangle.Y; }
            set { sourceRectangle.Y = value; }
        }

        public int SourceWidth
        {
            get { return sourceRectangle.Width; }
            set { sourceRectangle.Width = value; }
        }

        public int SourceHeight
        {
            get { return sourceRectangle.Height; }
            set { sourceRectangle.Height = value; }
        }

        public Texture2D Texture
        {
            get { return tex; }
            set
            {
                tex = value;
                textureNeedsUpdate = false;
            }
        }

        public string TextureName
        {
            get { return textureName; }
            set
            {
                if (this.beforeTexName != null) this.beforeTexName(out value);
                textureName = value;
                if (this.afterTexName != null) this.afterTexName(out textureName);
                textureNeedsUpdate = true;
            }
        }

        public GUIWidget()
            : base()
        {
        }

        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public virtual void Update(ZiderGame game, GameTime gameTime)
        {
            // TODO: Add your update code here
            if (this.getTex != null && textureNeedsUpdate)
            {
                Texture2D t;
                if (this.getTex(textureName, out t))
                {
                    this.tex = t;
                    textureNeedsUpdate = false;
                }
            }

            // update any children  
            foreach (Interfaces.IGUIWidget child in this.children)
            {
                //if (child.GetType() == typeof(Interfaces.IGUIWidget)) { 
                //Interfaces.IGUIWidget gw = (Interfaces.IGUIWidget)child;
                //gw.Update(game, gameTime);
                child.Update(game, gameTime);
                //         }
            }
        }

        /// <summary>
        /// Allows the game component to draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public virtual void Draw(GameTime gameTime, SpriteBatch sb)
        {
            this.Draw(this, gameTime, sb);
        }

        protected virtual void Draw(Interfaces.IGUIWidget c, GameTime gameTime, SpriteBatch sb)
        {
            if (!this.Visible) { return; }

            if (this.tex != null)
            {
                // calc dest rect
                Vector2 v = new Vector2(this.Left, this.Top);
                if (this.Parent != null)
                {
                    v = this.Parent.PointToScreen(new Vector2(this.Left, this.Top));
                }
                Rectangle dr = new Rectangle((int)v.X, (int)v.Y, this.Width, this.Height);

                // draw control
                sb.Draw(this.Texture, dr, this.sourceRectangle, this.color);
            }

            // draw text
            if (this.font != null && this.text != string.Empty)
            {
                // save bounding box
                Rectangle bb = this.font.BoundingBox;
               
                // get min max points
                Vector2 min, max;
                min = this.PointToScreen(0, 0);
                max = this.PointToScreen(this.Width, this.Height);
                           
                // set new temporary bounding box
               // this.font.BoundingBox = new BoundingBox(new Vector3(min.X, min.Y, 0), new Vector3(max.X, max.Y, 0));
                // draw text
                this.font.DrawString(Vector2.Zero, this.text, this.textAlignment, this.textColor);
                // restore bounding box
               // this.font.BoundingBox = bb;
            }

            // draw any children  
            foreach (Interfaces.IGUIWidget child in this.children)
            {
                //Classes.GUIWidget gw = (GUIWidget)child;
                //gw.Draw(gameTime, sb);
                child.Draw(gameTime, sb);
            }
        }



    }
}


