/*
Created by: X
http://www.createdbyx.com/

The Zider engine is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/
*/
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace Zider
{
    public class Map :IDisposable
    {
       // internal protected Rectangle playArea = new Rectangle(0, 0, 480, 480);
        internal protected Block[,] blocks;
        internal protected float blockWidth = 16;
        internal protected float blockHeight = 16;
        internal protected int mapWidth = 20;
        internal protected int mapHeight = 10;
        internal string mapTitle = string.Empty;
        internal protected bool needsToBeInit = true;
        internal ZiderGame game;
        internal protected Color backgroundColor = Color.CornflowerBlue;
        internal string mapFile;
        internal protected Collections.TextureDictionary textures = new Zider.Collections.TextureDictionary();

        public Collections.TextureDictionary Textures
        {
            get { return this.textures; }
        }

        public string MapFile
        {
            get { return mapFile; }
        }
	
        public Map()
        {
            Zider.Properties.Settings sett = Zider.Properties.Settings.Default;
            this.backgroundColor = new Color(sett.DefaultBackgroundColorR, 
                                             sett.DefaultBackgroundColorG, 
                                             sett.DefaultBackgroundColorB);
            this.blockWidth = sett.DefaultBlockWidth;
            this.blockHeight = sett.DefaultBlockHeight;
            this.mapWidth = sett.DefaultMapWidth;
            this.mapHeight = sett.DefaultMapHeight;
        }

        public Color BackgroundColor
        {
            get { return backgroundColor; }
            set { backgroundColor = value; }
        }
	
        public bool NeedsToBeInitilized
        {
            get { return needsToBeInit; }
        }

        public string MapTitle
        {
            get { return mapTitle; }
            set { mapTitle = value; }
        }
	
        public float BlockWidth
        {
            get { return blockWidth; }
           // set { blockWidth = value; }
        }

        public float BlockHeight
        {
            get { return blockHeight; }
           // set { blockHeight = value; }
        }

        public int MapWidth
        {
            get { return mapWidth; }
            set
            {
                mapWidth = value;
                this.needsToBeInit = true;
            }
        }

        public int MapHeight
        {
            get { return mapHeight; }
            set { 
                mapHeight=value;
                this.needsToBeInit = true;
            }
        }

        //public Rectangle PlayArea
        //{
        //    get { return PlayArea; }
        //    set { playArea = value; }
        //}

        //public int PlayAreaLeft
        //{
        //    get { return PlayArea.X; }
        //    set { playArea.X = value; }
        //}

        //public int PlayAreaTop
        //{
        //    get { return PlayArea.Y; }
        //    set { playArea.Y = value; }
        //}

        //public int PlayAreaWidth
        //{
        //    get { return PlayArea.Width; }
        //    set { playArea.Width = value; }
        //}

        //public int PlayAreaHeight
        //{
        //    get { return PlayArea.Height; }
        //    set { playArea.Height = value; }
        //}

        public Map(ZiderGame game)
        {
            this.Initilize();
            this.game = game;
        }

        public void Initilize()
        {
            // TODO: need to add option for preserving existing block data
           // mapWidth = (int)(playArea.Width / blockWidth);
            //mapHeight = (int)(playArea.Height / blockHeight);
            blocks = new Block[mapWidth, mapHeight];  
        }
                  
        public Block GetBlock(int x, int y)
        {
            return blocks[x, y]; 
        }

        public void SetBlock(int x, int y, Block value)
        {
            if (x < 0 || x > this.mapWidth - 1) { return; }
            if (y < 0 || y > this.mapHeight - 1) { return; }
            blocks[x, y] = value;
            if (value != null && value.GetTexture == null)
            {
                value.GetTexture = this.game.GetGameTexture;                
            }
        }

        public void Resize(int w, int h)
        {
            if (w < 1) throw new ArgumentOutOfRangeException("w");
            if (h < 1) throw new ArgumentOutOfRangeException("h");
            this.mapWidth = w;
            this.mapHeight = h;
            this.needsToBeInit = true;
        }

        #region IDisposable Members

        public void Dispose()
        {
            foreach (Collections.TextureInfo item in this.textures.Values)
            {
                item.Texture.Dispose();
            }
            this.textures.Clear();
            this.textures = null;
            this.blocks = null;
        }

        #endregion
    }
}
