/*
Created by: X
http://www.createdbyx.com/

The Zider engine is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/
*/
using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Zider.Properties;

namespace Zider.Classes
{
    public class MapSaver
    {
        private Map map;
        private ZiderGame game;

        public Map Map
        {
            get { return map; }
            set { map = value; }
        }

        public MapSaver(ZiderGame e, Map m)
        {
            this.map = m;
            this.game = e;
        }

        public bool Save(string file)
        {
            XmlDocument doc = new XmlDocument();
            doc.LoadXml("<?xml version=\"1.0\" encoding=\"utf-8\" ?>\n<map>\n</map>");

            XmlNode root = doc.DocumentElement;

            this.game.Console.Print(Resources.SavingMapHeader + Resources.DottedPause);
            // save map info
            CreateNode(root, Resources.XML_title, this.map.MapTitle);
            CreateNode(root, Resources.XML_mapwidth, this.map.MapWidth.ToString());
            CreateNode(root, Resources.XML_mapheight, this.map.MapHeight.ToString());
            CreateNode(root, Resources.XML_blockwidth, this.map.BlockWidth.ToString());
            CreateNode(root, Resources.XML_blockheight, this.map.BlockHeight.ToString());

            this.game.Console.Print(Resources.DottedPause + Resources.Done);
            this.game.Console.Print(Resources.SavingTextures + Resources.DottedPause);
            SaveTextures(root);

            this.game.Console.Print(Resources.DottedPause + Resources.Done);
            this.game.Console.Print(Resources.SavingBlocks + Resources.DottedPause);
            SaveBlocks(root);
            this.game.Console.Print(Resources.DottedPause + Resources.Done);

            try
            {
                this.game.Console.Print(Resources.SavingFile + Resources.DottedPause);
                doc.Save(file);
                this.game.Console.Print(Resources.DottedPause + Resources.SaveSucsessfull);
            }
            catch (Exception)
            {
                this.game.Console.Print(Resources.DottedPause + Resources.SaveNotSucsessfull);                
            } 
            
            doc = null;

            return true;
        }

        private void SaveBlocks(XmlNode root)
        {
            XmlNode blocks = CreateNode(root, Resources.XML_blocks, string.Empty);
            for (int idy = 0; idy <= this.map.MapHeight - 1; idy++)
            {
                for (int idx = 0; idx <= this.map.MapWidth - 1; idx++)
                {
                    Block b;
                    b = this.map.GetBlock(idx, idy);
                    if (b != null)
                    {
                        XmlNode n;
                        n = CreateNode(blocks, Resources.XML_block, string.Empty);
                        CreateNode(n, Resources.XML_type, "Platform");
                        CreateNode(n, Resources.XML_x, idx.ToString());
                        CreateNode(n, Resources.XML_y, idy.ToString());
                        CreateNode(n, Resources.XML_sourceleft, b.SourceLeft.ToString());
                        CreateNode(n, Resources.XML_sourcetop, b.SourceTop.ToString());
                        CreateNode(n, Resources.XML_sourcewidth, b.SourceWidth.ToString());
                        CreateNode(n, Resources.XML_sourceheight, b.SourceHeight.ToString());
                        CreateNode(n, Resources.XML_texture, b.TextureName);
                    }
                }
            }
        }

        private void SaveTextures(XmlNode root)
        {
            XmlNode textures = CreateNode(root, Resources.XML_textures, string.Empty);
            foreach (string key in this.game.CurrentMap.Textures.Keys)
            {
                Collections.TextureInfo info;
                info = this.game.CurrentMap.Textures[key];
                XmlNode n;
                n = CreateNode(textures, Resources.XML_texture, string.Empty);
                CreateNode(n, Resources.XML_name, info.TextureName);
                CreateNode(n, Resources.XML_file, info.SourceFile);
            }
        }

        private XmlNode CreateNode(XmlNode parent, string name, string value)
        {
            XmlElement n = parent.OwnerDocument.CreateElement(name);
            if (value != string.Empty) n.AppendChild(parent.OwnerDocument.CreateTextNode(value));
            parent.AppendChild(n);
            return n;
        }

    }
}
