/*
Created by: X
http://www.createdbyx.com/

The Zider engine is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/
*/
using System;
using System.Collections.Generic;
using System.Text;
using System.Reflection;
using Microsoft.Xna.Framework;
using Zider.Properties;
using xFripperyXNA;
using Microsoft.Xna.Framework.Graphics;

namespace Zider.Classes.Widgets
{
    public class ButtonTexturesWidget : Interfaces.IDesktopComponent
    {
        private Classes.GUIWidget widget;
        private Classes.GUIWidget leftWidget;
        private Classes.GUIWidget rightWidget;
        private Components.TextureSelector textureSelector;


        public int Left
        {
            get { return this.widget.Left; }
            set { this.widget.Left = value; }
        }

        public int Top
        {
            get { return this.widget.Top; }
            set { this.widget.Top = value; }
        }

        public int Width
        {
            get { return this.widget.Width; }
            set { this.widget.Width = value; }
        }

        public int Height
        {
            get { return this.widget.Height; }
            set { this.widget.Height = value; }
        }
        
        public Classes.GUIWidget Widget
        {
            get { return this.widget; }
        }

        public Rectangle RightSourceRectangle
        {
            get { return this.rightWidget.SourceRectangle; }
            set { this.rightWidget.SourceRectangle = value; }
        }

        public Rectangle LeftSourceRectangle
        {
            get { return this.leftWidget.SourceRectangle; }
            set { this.leftWidget.SourceRectangle = value; }
        }

        public string RightTexture
        {
            get { return this.rightWidget.TextureName; }
            set { this.rightWidget.TextureName = value; }
        }

        public string LeftTexture
        {
            get { return this.leftWidget.TextureName; }
            set { this.leftWidget.TextureName = value; }
        }


        public Classes.GUIWidget LeftWidget
        {
            get { return leftWidget; }

            set
            {
                if (this.leftWidget != null) { this.leftWidget.Click -= new xFripperyXNA.ClickEventHandler(LeftButton_Click); }
                leftWidget = value;
                if (this.leftWidget != null) { this.leftWidget.Click += new xFripperyXNA.ClickEventHandler(LeftButton_Click); }
            }
        }

        public Classes.GUIWidget RightWidget
        {
            get { return rightWidget; }
            set
            {
                if (this.rightWidget != null) { this.rightWidget.Click -= new xFripperyXNA.ClickEventHandler(RightButton_Click); }
                rightWidget = value;
                if (this.rightWidget != null) { this.rightWidget.Click += new xFripperyXNA.ClickEventHandler(RightButton_Click); }
            }
        }

        public ButtonTexturesWidget(Classes.GUIWidget w)
        {
            if (w == null) throw new ArgumentNullException("w");
            this.widget = w;    
            // search for left and right texture button controls
            this.LeftWidget = this.widget.Children.FindControl<Classes.GUIWidget>("leftbutton", true, true);
            this.RightWidget = this.widget.Children.FindControl<Classes.GUIWidget>("rightbutton", true, true);

            Texture2D tex = this.widget.Game.guiTextures[Resources.NoTexture].Texture;
            this.leftWidget.TextureName = string.Empty;
            this.rightWidget.TextureName = string.Empty;
            this.rightWidget.SourceRectangle = new Rectangle(0, 0, tex.Width, tex.Height);
            this.leftWidget.SourceRectangle = new Rectangle(0, 0, tex.Width, tex.Height);

            this.leftWidget.Texture = tex;
            this.rightWidget.Texture = tex;  
        }

        void RightButton_Click(object sender, MouseButtons b, Vector2 pos)
        {
            if (sender == null) return;
            if (sender is Commands.GUIInput)
            {
                Commands.GUIInput i = (Commands.GUIInput)sender;
                if (b == MouseButtons.Right)
                {
                    Texture2D tex = this.widget.Game.guiTextures[Resources.NoTexture].Texture;
                    this.rightWidget.SourceRectangle = new Rectangle(0, 0, tex.Width, tex.Height);
                    this.rightWidget.TextureName = string.Empty;
                    this.rightWidget.Texture = tex;
                }
                else
                {
                    if (textureSelector == null)
                    {
                        textureSelector = new Zider.Components.TextureSelector(this.widget.Game);
                        textureSelector.Canceled += new Zider.Components.TextureSelector.CanceledEventHandler(TextureSelection_Canceled);
                        this.leftWidget.GetTexture = this.widget.Game.GetGameTexture;
                        this.rightWidget.GetTexture = this.widget.Game.GetGameTexture;
                    }
                    textureSelector.SelectedTexture += new Zider.Components.TextureSelector.SelectedTextureEventHandler(Right_SelectedTexture);
                    textureSelector.Show();
                }
            }
        }

        void LeftButton_Click(object sender, MouseButtons b, Vector2 pos)
        {
            if (sender == null) return;
            if (sender is Commands.GUIInput)
            {
                Commands.GUIInput i = (Commands.GUIInput)sender;
                if (b == MouseButtons.Right)
                {
                    Texture2D tex = this.widget.Game.guiTextures[Resources.NoTexture].Texture;
                    this.leftWidget.SourceRectangle = new Rectangle(0, 0, tex.Width, tex.Height);
                    this.leftWidget.TextureName = string.Empty;
                    this.leftWidget.Texture = tex;
                }
                else
                {
                    if (textureSelector == null)
                    {
                        textureSelector = new Zider.Components.TextureSelector(this.widget.Game);
                        textureSelector.Canceled += new Zider.Components.TextureSelector.CanceledEventHandler(TextureSelection_Canceled);
                        this.leftWidget.GetTexture = this.widget.Game.GetGameTexture;
                        this.rightWidget.GetTexture = this.widget.Game.GetGameTexture;
                    }
                    textureSelector.SelectedTexture += new Zider.Components.TextureSelector.SelectedTextureEventHandler(Left_SelectedTexture);
                    textureSelector.Show();
                }
            }
        }

        void TextureSelection_Canceled()
        {
            textureSelector.SelectedTexture -= new Zider.Components.TextureSelector.SelectedTextureEventHandler(Left_SelectedTexture);
            textureSelector.SelectedTexture -= new Zider.Components.TextureSelector.SelectedTextureEventHandler(Right_SelectedTexture);
        }

        void Left_SelectedTexture(string Name, Rectangle sourceRectangle)
        {
            textureSelector.SelectedTexture -= new Zider.Components.TextureSelector.SelectedTextureEventHandler(Left_SelectedTexture);
            this.leftWidget.TextureName = Name;
            this.leftWidget.SourceRectangle = sourceRectangle;
        }

        void Right_SelectedTexture(string Name, Rectangle sourceRectangle)
        {
            textureSelector.SelectedTexture -= new Zider.Components.TextureSelector.SelectedTextureEventHandler(Right_SelectedTexture);
            this.rightWidget.TextureName = Name;
            this.rightWidget.SourceRectangle = sourceRectangle;
        }


        #region IDesktopComponent Members

        void Zider.Interfaces.IDesktopComponent.Update(ZiderGame game, GameTime gameTime)
        {
            this.widget.Update(game, gameTime);
        }

        void Zider.Interfaces.IDesktopComponent.Draw(GameTime gameTime, SpriteBatch sb)
        {
            this.widget.Draw(gameTime, sb);
        }

        GUIWidget Zider.Interfaces.IDesktopComponent.Widget
        {
            get { return this.widget; }
        }

        #endregion
    }
}
