/*
Created by: X
http://www.createdbyx.com/

The Zider engine is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/
*/
#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
#endregion

namespace Zider.Collections
{

    public class TextureInfo
    {
        protected Texture2D tex;
        protected string texName;
        protected string sourceFile;
        protected bool textureNeedsUpdate = true;
        protected ZiderGame game;


        public virtual bool TextureNeedsUpdate
        {
            get { return textureNeedsUpdate; }
        }

        public virtual string SourceFile
        {
            get { return sourceFile; }
            set
            {
                sourceFile = value;
                this.textureNeedsUpdate = true;
            }
        }

        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public virtual void Update(GameTime gameTime)
        {
            // TODO: Add your update code here
            string file = System.IO.Path.Combine(Environment.CurrentDirectory, this.SourceFile);
            if (textureNeedsUpdate && System.IO.File.Exists(file))
            {
                Texture2D t;
                //this.sourceFile = System.IO.Path.GetFullPath(this.sourceFile);
                t = Texture2D.FromFile(this.game.GraphicsDevice, file);
                this.tex = t;
                textureNeedsUpdate = false;
            }
        }

        public virtual string TextureName
        {
            get { return texName; }
            set { texName = value; }
        }

        public virtual Texture2D Texture
        {
            get { return tex; }
            set { tex = value; }
        }

        public TextureInfo(ZiderGame g)
        {
            this.game = g;
        }

        public TextureInfo(ZiderGame g, string file, string name, Texture2D t)
        {
            this.game = g;
            this.sourceFile = file;
            this.texName = name;
            this.textureNeedsUpdate = false;
            this.tex = t;
        }

        public TextureInfo(ZiderGame g, string file, string name)
        {
            this.game = g;
            this.sourceFile = file;
            this.textureNeedsUpdate = true;
            this.texName = name;
        }

        public TextureInfo(Texture2D t)
        {
            this.tex = t;
            this.textureNeedsUpdate = false;
        }

        public TextureInfo(Texture2D t, string file)
        {
            this.tex = t;
            this.sourceFile = file;
            this.textureNeedsUpdate = false;
        }
    }

    public class TextureDictionary : Dictionary<string, TextureInfo>
    {

    }
}
