/*
Created by: X
http://www.createdbyx.com/

The Zider engine is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/
*/

#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Graphics;
using Zider.Properties;

#endregion

namespace Zider.Commands
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public class BlockEditor : Interfaces.ICommand
    {

        internal protected Components.InputManager inputManager;
        internal protected string firstTexture = string.Empty;
        internal protected string secondTexture = string.Empty;
        private BlockType firstBlockType = BlockType.Platform;
        private BlockType secondBlockType = BlockType.Platform;
        private Interfaces.CommandResult result;
        private Classes.Widgets.ButtonTexturesWidget bt;


        public Interfaces.CommandResult Result
        {
            get { return result; }
        }

        public BlockType SecondBlockType
        {
            get { return secondBlockType; }
            set { secondBlockType = value; }
        }

        public BlockType FirstBlockType
        {
            get { return firstBlockType; }
            set { firstBlockType = value; }
        }

        public string FirstTexture
        {
            get { return firstTexture; }
            set { firstTexture = value; }
        }

        public string SecondTexture
        {
            get { return secondTexture; }
            set { secondTexture = value; }
        }

        public BlockEditor(Components.InputManager im)
        {
            // TODO: Construct any child components here
            this.inputManager = im;

        }


        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public void Initialize()
        {
            // TODO: Add your initialization code here

        }

        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public void Update(GameTime gameTime)
        {
            if (bt == null)
            {
                Interfaces.IDesktopComponent w;
                if (!this.inputManager.Engine.GUI.Desktop.TryGetValue(Resources.ButtonTextures, out w))
                {
                    throw new NullReferenceException(Resources.CouldNotFindBTW);
                }
                bt = (Classes.Widgets.ButtonTexturesWidget)w;
            }

            MouseState state = Mouse.GetState();
            // check if drawing with left mouse button
            if (state.LeftButton == ButtonState.Pressed)
            {                   
                DrawBlock(firstBlockType, bt.LeftTexture, bt.LeftSourceRectangle);
                this.result = Zider.Interfaces.CommandResult.Handled;
            }

            // check if drawing with left mouse button
            if (state.RightButton == ButtonState.Pressed)
            {
                DrawBlock(secondBlockType, bt.RightTexture, bt.RightSourceRectangle);
                this.result = Zider.Interfaces.CommandResult.Handled;
            }
        }

        private void DrawBlock(BlockType type, string t, Rectangle sr)
        {
            ZiderGame e = this.inputManager.game;
            // calc block position
            int x = (int)((e.pointerPosition.X + e.cam.Position.X) / e.CurrentMap.blockWidth);
            int y = (int)((e.pointerPosition.Y + e.cam.Position.Y) / e.CurrentMap.blockHeight);
            Block b = null;
            if (t != string.Empty && t != null)
            {
                b = Helpers.CreateBlock(type, sr, t);
            }

            // set the block within the map
            e.CurrentMap.SetBlock(x, y, b);
        }
    }
}


