/*
Created by: X
http://www.createdbyx.com/

The Zider engine is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/
*/
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;
using Zider.Properties;

namespace Zider.Commands
{
    public class GUIInput : Interfaces.ICommand
    {
        internal protected Components.InputManager inputManager;
        internal protected xFripperyXNA.xFrippery inputHandler = new xFripperyXNA.xFrippery();
        private MouseState prevMouseState;
        private KeyboardState prevKBState;
        private Interfaces.CommandResult result;
        private ZiderGame game;


        public Interfaces.CommandResult Result
        {
            get { return result; }
        }

        public Components.InputManager InputManager
        {
            get { return inputManager; }
        }

        public GUIInput(Components.InputManager im)
        {
            if (im == null) { throw new ArgumentNullException("im"); }
            this.inputManager = im;
            this.game = im.Engine;
        }

        #region ICommand Members

        public void Initialize()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void Update(GameTime gameTime)
        {

            // update pointer pos
            MouseState state = Mouse.GetState();
            KeyboardState kbstate = Keyboard.GetState();

            this.game.PointerPosition = new Vector2(state.X, state.Y);

            // if the console is visible allow it to operate
            if (this.game.Console.Visible)
            {
                this.result = Zider.Interfaces.CommandResult.Handled;
                return;
            }

            // set default result
            Zider.Interfaces.CommandResult res = Zider.Interfaces.CommandResult.Continue;
            foreach (string key in this.inputManager.game.GUI.Desktop.Keys)
            {
                // get desktop component
                Interfaces.IDesktopComponent c;
                c = this.inputManager.game.GUI.Desktop[key];

                // setup desktop component to handle input for
                inputHandler.Controls.Clear();
                inputHandler.Controls.Add(c.Widget);

                // process mouse state
                ProcessMouse(gameTime, state);

                // process keyboard state
                res = ProcessKeyboard(gameTime, kbstate);
            }

            // store result
            this.result = res;
        }

        private void ProcessMouse(GameTime gameTime, MouseState state)
        {
            xFripperyXNA.MouseButtons btns = xFripperyXNA.MouseButtons.None;
            bool changed = false;

            // apply panning
            PanMap(gameTime);

            // get buttons that were pressed
            if (state.LeftButton == ButtonState.Pressed && prevMouseState.LeftButton == ButtonState.Released)
            {
                changed = true;
                btns = btns | xFripperyXNA.MouseButtons.Left;
            }
            if (state.RightButton == ButtonState.Pressed && prevMouseState.RightButton == ButtonState.Released)
            {
                changed = true;
                btns = btns | xFripperyXNA.MouseButtons.Right;
            }
            // call mouse down if buttons were pressed
            if (changed) { inputHandler.DoMouseDown(this, btns, inputManager.game.PointerPosition); }

            // get change in position
            Vector2 pos;
            bool poschanged = false;
            if (prevMouseState.X != state.X)
            {
                poschanged = true;
                pos.X = state.X - prevMouseState.X;
            }
            if (prevMouseState.Y != state.Y)
            {
                poschanged = true;
                pos.Y = state.Y - prevMouseState.Y;
            }
            // clear the buttons varible
            btns = xFripperyXNA.MouseButtons.None;

            // get buttons that were released
            if (prevMouseState.LeftButton == ButtonState.Pressed && state.LeftButton == ButtonState.Released)
            {
                changed = true;
                btns = btns | xFripperyXNA.MouseButtons.Left;
            }
            if (prevMouseState.RightButton == ButtonState.Pressed && state.RightButton == ButtonState.Released)
            {
                changed = true;
                btns = btns | xFripperyXNA.MouseButtons.Right;
            }

            if (poschanged) { inputHandler.DoMouseMove(this, btns, inputManager.game.PointerPosition); }
            if (changed) { inputHandler.DoMouseUp(this, btns, inputManager.game.PointerPosition); }

            // save state
            this.prevMouseState = state;
        }

        private void PanMap(GameTime gameTime)
        {
            ZiderGame e = this.game;
            Rectangle r = e.cam.ViewportRectangle;
            // deflate the rect by 3%
            r.Inflate((int)-(r.Width * 0.03f), (int)-(r.Height * 0.03f));
            // check if mouse is NOT in deflated rect
            if (e.pointerPosition.X < r.Left || e.pointerPosition.Y < r.Top ||
                e.pointerPosition.X > r.Right || e.pointerPosition.Y > r.Bottom)
            {
                // calc cursor vector on screen
                Vector3 v = new Vector3(r.Left + (r.Width / 2), r.Top + (r.Height / 2), 0) - new Vector3(e.PointerPosition.X, e.pointerPosition.Y, 0);
                // normalize vector
                v.Normalize();

                // calc speed to pan
                v *= -0.5f * gameTime.ElapsedGameTime.Milliseconds;

                // pan map by moving camera
                e.cam.Move(v);
            }
        }

        private Zider.Interfaces.CommandResult ProcessKeyboard(GameTime gameTime, KeyboardState state)
        {

            Zider.Properties.Settings sett = Zider.Properties.Settings.Default;

            // check if showing console
            if (state.IsKeyUp(sett.ConsoleKeyToggle) && prevKBState.IsKeyDown(sett.ConsoleKeyToggle))
            {
                if (!this.game.Console.Visible)
                {
                    this.game.Console.Console.Entryline = string.Empty;
                    this.game.Console.Show();
                    // save state
                    this.prevKBState = state;
                    return Zider.Interfaces.CommandResult.Handled;
                }
            }

            // check if paning map
            PanMapKB(state, gameTime);

            // check to reset cam pos
            if (state.IsKeyUp(sett.ResetCameraKey) && prevKBState.IsKeyDown(sett.ResetCameraKey))
            {
                this.InputManager.game.cam.Position = Vector3.Backward;
                this.InputManager.game.cam.Direction = Vector3.Zero;
            }

            // check if toggling fullscreen
            if (state.IsKeyUp(sett.FullScreenToggleKey) && prevKBState.IsKeyDown(sett.FullScreenToggleKey))
            {
                this.InputManager.game.graphics.ToggleFullScreen();
            }

            // check if saving
            if (state.IsKeyUp(sett.SaveMapKey) && prevKBState.IsKeyDown(sett.SaveMapKey))
            {
                // create a map saver object
                Classes.MapSaver ms = new Zider.Classes.MapSaver(this.game, this.game.CurrentMap);
                bool saved = false;
                // get full filename and path
                string file = this.game.CurrentMap.MapFile;
                file = System.IO.Path.GetFullPath(file);
                // try to save file
                saved = ms.Save(file);
                // crop unnessary path info
                file = file.Replace(System.IO.Path.GetDirectoryName(this.game.fileName), string.Empty);
                // check if saved
                if (saved)
                {
                    // write to console that file was saved
                    this.game.Console.Print(Resources.SavedFile + file);
                }
                else
                {
                    // write to console file was not saved
                    this.game.Console.Print(Resources.CouldNotSaveFileName.Replace(Resources.RKEY, file));
                }
                ms = null;
            }

            // check if toggling grid
            if (state.IsKeyUp(sett.GridToggleKey) && prevKBState.IsKeyDown(sett.GridToggleKey))
            {
                Components.GridRenderer gr;
                gr = Helpers.FindComponent<Components.GridRenderer>(this.game.Components);
                if (state.IsKeyDown(Keys.LeftShift) | state.IsKeyDown(Keys.RightShift))
                {
                    switch (gr.GridStyle)
                    {
                        case Zider.Components.GridRenderer.GridType.Points:
                            gr.GridStyle = Zider.Components.GridRenderer.GridType.Lines;
                            break;

                        case Zider.Components.GridRenderer.GridType.Lines:
                            gr.GridStyle = Zider.Components.GridRenderer.GridType.Points;
                            break;
                    }

                }
                else
                {
                    gr.Visible = !gr.Visible;
                }
            }


            // check if toggling mapbounds
            if (state.IsKeyUp(sett.MapBoundsToggleKey) && prevKBState.IsKeyDown(sett.MapBoundsToggleKey))
            {
                Components.RectangleRenderer rr;
                rr = Helpers.FindComponent<Components.RectangleRenderer>(this.game.Components);
                Components.RectangleRenderer.RectangleInfo ri;
                ri = rr.NamedRectangles[Resources.MapBoundsKey];
                ri.Visible = !ri.Visible;
            }

            // save state
            this.prevKBState = state;

            return Zider.Interfaces.CommandResult.Continue;
        }

        private void PanMapKB(KeyboardState state, GameTime gameTime)
        {
            ZiderGame e = this.game;
            Vector3 v = Vector3.Zero;

            Zider.Properties.Settings sett = Zider.Properties.Settings.Default;
            // calc cursor vector on screen
            if (state.IsKeyDown(sett.PanUpKey)) { v += Vector3.Up; }
            if (state.IsKeyDown(sett.PanDownKey)) { v += Vector3.Down; }
            if (state.IsKeyDown(sett.PanLeftKey)) { v += Vector3.Right; }
            if (state.IsKeyDown(sett.PanRightKey)) { v += Vector3.Left; }

            // v -=  new Vector3(e.PointerPosition.X, e.pointerPosition.Y, 0);

            // calc speed to pan
            v *= -0.5f * gameTime.ElapsedGameTime.Milliseconds;

            // pan map by moving camera
            e.cam.Move(v);

        }



        #endregion
    }
}
