
#region Using Statements
using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Zider.Properties;
#endregion

namespace Zider.Components
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public partial class GameLoader : Microsoft.Xna.Framework.GameComponent
    {

        public enum LoadSteps
        {
            LoadFile,
            ParseTitle,
            ParseDescription,
           // ParseStartMap,
            ParseGameScript,
          //  ParseMaps,
            InitGame,
           // FadeOut,
           // LoadGUI,
            Done
        }

        protected ZiderGame game;
        protected SpriteBatch sb;
        protected string gameFile = string.Empty;
        protected LoadSteps currentStep = LoadSteps.InitGame;
        System.Xml.XmlDocument doc;
        protected string path;
        //private float fadeValue = 1;
        //private float fadeSpeed = 0.001f;
        //private bool fadeOut = true;
        //private Color fontColor = Color.White;
        List<IGameComponent> gameComponentList;
        private bool hasBeenInit = false;
        //private int lineSpacing = 4;
        private StringCollection gameScript = new StringCollection();


        //public int LineSpacing
        //{
        //    get { return lineSpacing; }
        //    set { lineSpacing = value; }
        //}

        //public void ResetFade()
        //{
        //    fadeValue = 1;
        //}

        //public Color FontColor
        //{
        //    get { return fontColor; }
        //    set { fontColor = value; }
        //}

        //public bool FadeOut
        //{
        //    get { return fadeOut; }
        //    set { fadeOut = value; }
        //}

        //public float FadeSpeed
        //{
        //    get { return fadeSpeed; }
        //    set { fadeSpeed = Math.Max(0, Math.Abs(value)); }
        //}

        public GameLoader(ZiderGame game, string gameFile)
            : base(game)
        {
            // TODO: Construct any child components here
            this.game = (ZiderGame)game;
            this.gameFile = gameFile;
            this.path = System.IO.Path.GetFullPath(gameFile);
            this.path = System.IO.Path.GetDirectoryName(this.path);
            this.gameFile = System.IO.Path.GetFileName(gameFile);
            Environment.CurrentDirectory = this.path;
        }


        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {

            if (!this.hasBeenInit)
            {
                // TODO: Add your initialization code here
                this.sb = new SpriteBatch(game.GraphicsDevice);
                this.doc = new System.Xml.XmlDocument();

                // get a list of all game components and remove them from the game components list
                // so we can restore then later
                this.gameComponentList = Helpers.HoldComponents(this.game, true);

                // make sure we have not stored this map loading component to the list
                this.gameComponentList.Remove(this);

                this.hasBeenInit = true;

                // re add this map loading component to the list
                this.game.Components.Add(this);
            }

            base.Initialize();
        }
                      

        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {
            // TODO: Add your update code here
            switch (currentStep)
            {
                case LoadSteps.InitGame:
                    // add the game components
                    AddGameComponents();
                    this.game.Console.Print(Resources.InitComponents + Resources.DottedPause);

                    currentStep = LoadSteps.LoadFile;
                    break;
                  
                //case LoadSteps.LoadGUI:
                //    this.game.Console.Print(Resources.LoadingDesktopWidgets + Resources.DottedPause);
                //    // load desktop widgets
                //    LoadDesktopGUI();
                //    this.game.Console.Print(Resources.DottedPause + " " + Resources.Done);
                //    currentStep = LoadSteps.LoadFile;
                //    break;

                case LoadSteps.LoadFile:
                    this.game.Console.Print(Resources.LoadingFile + " " + gameFile);

                    try
                    {
                        doc.Load(System.IO.Path.Combine(this.path, gameFile));
                        if (doc.DocumentElement.Name != Resources.XML_game)
                        {
                            //ERR: Need to handle this exception better
                            throw new System.IO.InvalidDataException(Resources.FileIsNotAValidGameFile);
                        }
                    }
                    catch (Exception)
                    {
                        throw;
                    }

                    // set the next step
                    currentStep = LoadSteps.ParseTitle;
                    break;

                case LoadSteps.ParseTitle:
                    ParseStringNode(doc.DocumentElement, Resources.XML_title, out game.gameTitle);

                    this.game.Console.Print(Resources.GameTitle + game.GameTitle);
                    game.Window.Title = this.game.GameTitle;
                    currentStep = LoadSteps.ParseDescription;
                    break;

                case LoadSteps.ParseDescription:
                    try
                    {
                        ParseStringNode(doc.DocumentElement, Resources.XML_description, out game.gameDescription);
                    }
                    catch (Exception)
                    {
                        // if no description found none will be used
                        // description for game file is optional
                    }
                    this.game.Console.Print(Resources.GameDescription + game.GameDescription);
                    currentStep = LoadSteps.ParseGameScript;
                    break;

                //case LoadSteps.ParseStartMap:
                //    ParseStringNode(doc.DocumentElement, Resources.XML_startmap, out game.startMap);

                //    this.game.Console.Print(Resources.StartingMap + game.startMap);
                //    currentStep = LoadSteps.ParseGameScript;
                //    break;

                case LoadSteps.ParseGameScript:
                    this.game.Console.Print(Resources.ProcessingScript + Resources.DottedPause);
                    Helpers.ProcessScript(this.gameScript, System.IO.Path.Combine(this.path, gameFile), doc.DocumentElement, true);
                    
                    this.game.Console.Print(Resources.DottedPause + Resources.ScriptProcessed);
                    currentStep = LoadSteps.Done;
                    break;
                
                //case LoadSteps.ParseMaps:
                //    ParseMapList(doc.DocumentElement);
                //    // report number of maps loaded into map list
                //    this.game.Console.Print(Resources.NumberOfMaps + game.MapList.Count.ToString());

                //    currentStep = LoadSteps.Done;
                //    break;

               
                //case LoadSteps.FadeOut:
                //    if (fadeOut)
                //    {
                //        this.fadeValue -= fadeSpeed * gameTime.ElapsedGameTime.Milliseconds;
                //        this.fadeValue = Math.Max(0, fadeValue);
                //        if (this.fadeValue == 0)
                //        {
                //            currentStep = LoadSteps.Done;
                //        }
                //    }
                //    else
                //    {
                //        currentStep = LoadSteps.Done;
                //    }
                //    break;

                case LoadSteps.Done:
                    // remove this compoent
                    this.game.Components.Remove(this);
                  
                    // restore all previous game components
                    Helpers.RestoreComponents(this.game, gameComponentList);

                    this.game.Console.Print(Resources.ExecutingGameScript);
                    // run game script
                    this.game.Console.Console.CommandManager.Execute(this.gameScript, null);
                    
                    break;
            }

            base.Update(gameTime);
        }

        //private void ParseMapList(System.Xml.XmlNode node)
        //{
        //    // try to get maps node
        //    System.Xml.XmlNode maps;
        //    maps = node.SelectSingleNode(Resources.XML_maps);
        //    // if no maps node then report it in console
        //    if (maps == null )
        //    {
        //        this.game.console.Console.Print(Resources.MissingNamedElement.Replace(Resources.RKEY, Resources.XML_maps));
        //        return;
        //    }
           
        //    // try to get map nodes
        //    System.Xml.XmlNodeList list;
        //    list = maps.SelectNodes(Resources.XML_map);
        //    foreach (System.Xml.XmlNode n in list)
        //    {
        //        // get map name and associated file then add to map list
        //        string name, file;
        //        ParseStringNode(n, Resources.XML_name, out name);
        //        ParseStringNode(n, Resources.XML_file, out file);
        //        this.game.MapList.Add(name, file);
        //    }
        //}

        private void AddGameComponents()
        {
            // add required components (in reverse order)
            // ------------------------------------------
            Zider.Properties.Settings sett = Zider.Properties.Settings.Default;
                
            // add console component
            game.console = new xGameConsoleXNA.ConsoleComponent(game);
            game.console.Console.ConsoleSpeed = 0.5f;
            game.console.Console.MaxConsolePercentage = 100;
            Rectangle r = Helpers.CenterRectangle(new Rectangle(0, 0, 640, 480), game.cam.ViewportWidth / 2, game.cam.ViewportHeight / 2);
            game.console.Bounds = r;
            game.console.LineSpacing = 4;
            game.console.Font = game.content.Load<Nuclex.Fonts.BitmapFont>(@Resources.ConsoleFont);
            game.Components.Add(game.console);
            game.console.ExtenderManager.Add(new xGameConsoleXNA.GenericCommands.GenericCommands(game.console.Console), true);


            // add game commands into console
            AddConsoleCommands();

            // add gui renderer
            game.gui = new Zider.Components.GUIRenderer(game);
            game.Components.Add(game.gui);

            // add the rectangle renderer that will draw rectangles on screen
            Components.RectangleRenderer rr = new Components.RectangleRenderer(game);
            int mw, mh;
            mw = (int)(game.CurrentMap.mapWidth * game.CurrentMap.blockWidth);
            mh = (int)(game.CurrentMap.mapHeight * game.CurrentMap.blockHeight);
            Color mbc = new Color(sett.DefaultMapBorderColorR, sett.DefaultMapBorderColorG,
                                  sett.DefaultMapBorderColorB, sett.DefaultMapBorderColorA);
            rr.NamedRectangles.Add(Resources.MapBoundsKey, new Components.RectangleRenderer.RectangleInfo(new Rectangle(0, 0, mw, mh), mbc, sett.MapBoundaryVisible));
            game.Components.Add(rr);

            //add the grid rendering component
            Color gc = new Color(sett.DefaultGridColorR, sett.DefaultGridColorG, sett.DefaultGridColorB, sett.DefaultGridColorA);
            Zider.Components.GridRenderer gr = new Components.GridRenderer(game, game.CurrentMap.blockWidth, game.CurrentMap.blockHeight);
            gr.GridColor = gc;
            gr.Visible = sett.GridVisible;
            game.Components.Add(gr);
            // add the map rendering component
            game.Components.Add(new Components.MapRenderer(game));
            // create the input manager component
            game.inputManager = new Components.InputManager(game);
            // add the controller manager
            game.Components.Add(game.controllerManager);
            // add the input manager component
            game.Components.Add(game.inputManager);
        }

        /// <summary>
        /// Adds standard console commands to the console
        /// </summary>
        private void AddConsoleCommands()
        {
            game.console.ExtenderManager.Add(new Console.StandardCommands(game), true);
        }
             

        private void ParseStringNode(System.Xml.XmlNode parentnode, string name, out string value)
        {
            System.Xml.XmlNode node = parentnode.SelectSingleNode(name);
            if (node == null) throw new System.IO.InvalidDataException(Resources.CouldNotFindNamedElement.Replace(Resources.RKEY, name));
            value = node.InnerText;
            return;
        }

        ///// <summary>
        ///// Allows the game component to draw itself.
        ///// </summary>
        ///// <param name="gameTime">Provides a snapshot of timing values.</param>
        //public override void Draw(GameTime gameTime)
        //{
        //    // TODO: Add your draw code here
        //    int y = this.GraphicsDevice.Viewport.Height; //-game.font.Height; ;
        //    Color col = new Color(fontColor.R, fontColor.G, fontColor.B, (byte)(fadeValue * byte.MaxValue));
        //    for (int idx = info.Count - 1; idx >= 0; idx--)
        //    {
        //        Rectangle sr;
        //        string txt;
        //        txt = info[idx];
        //        sr = game.DefaultFont.MeasureString(txt);
        //        y -= sr.Height + this.lineSpacing;
        //        game.DefaultFont.DrawString(new Vector2(0, y), txt, Nuclex.Fonts.TextAlignment.Left | Nuclex.Fonts.TextAlignment.Top, col);
        //    }

        //    base.Draw(gameTime);
        //}
    }
}


