/*
Created by: X
http://www.createdbyx.com/

The Zider engine is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/
*/

#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Zider.Properties;
#endregion

namespace Zider.Components
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public partial class MapLoader : Microsoft.Xna.Framework.GameComponent
    {

        public enum LoadSteps
        {
            LoadFile,
            ParseTitle,
            ParseBackgroundColor,
            ParseMapSize,
            ParseMapBlockSize,
            ParseTextures,
            ParseBlocks,
           // FadeOut,
            Done
        }

        protected ZiderGame game;
        protected SpriteBatch sb;
        protected string mapFile = string.Empty;
        protected LoadSteps currentStep = LoadSteps.LoadFile;
        System.Xml.XmlDocument doc;
       // protected string path;
        //protected System.Collections.Specialized.StringCollection info = new System.Collections.Specialized.StringCollection();
        //private float fadeValue = 1;
        //private float fadeSpeed = 0.001f;
        //private bool fadeOut = true;
        //private Color fontColor = Color.White;
      //  List<IGameComponent> gameComponentList;
        private bool hasBeenInit = false;
        //private int lineSpacing = 4;
       // private string mapName = string .Empty;        
        private Map map;

        //public int LineSpacing
        //{
        //    get { return lineSpacing; }
        //    set { lineSpacing = value; }
        //}
	


        //public void ResetFade()
        //{
        //    fadeValue = 1;
        //}

        //public Color FontColor
        //{
        //    get { return fontColor; }
        //    set { fontColor = value; }
        //}

        //public bool FadeOut
        //{
        //    get { return fadeOut; }
        //    set { fadeOut = value; }
        //}

        //public float FadeSpeed
        //{
        //    get { return fadeSpeed; }
        //    set { fadeSpeed = Math.Max(0, Math.Abs(value)); }
        //}

        //public float FadeValue
        //{
        //    get { return FadeValue; }
        //    set { FadeValue = Math.Abs(value); }
        //}

        public MapLoader(ZiderGame game, string mFile)
            : base(game)
        {
            // TODO: Construct any child components here
            this.game = game;
            //if (!this.game.MapList.ContainsKey(mFile))
            //{
            //    throw new KeyNotFoundException(Resources.MapNotInMapList.Replace(Resources.RKEY, mFile));
            //}
            this.mapFile = mFile;// this.game.MapList[mFile];
            //this.path = System.IO.Path.GetFullPath(this.mapFile);
            //this.path = System.IO.Path.GetDirectoryName(this.path);
            //this.mapFile = System.IO.Path.GetFullPath(this.mapFile);
            this.map = new Map(game);            
            // this.mapName = mFile;
            //Environment.CurrentDirectory = this.path;
        }


        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {

            if (!this.hasBeenInit)
            {
                // TODO: Add your initialization code here
                this.sb = new SpriteBatch(game.GraphicsDevice);
                this.doc = new System.Xml.XmlDocument();

                // get a list of all game components and remove them from the game components list
                // so we can restore then later
               // this.gameComponentList = Helpers.HoldComponents(this.game, true);

                // make sure we have not stored this map loading component to the list
               // this.gameComponentList.Remove(this);

                this.hasBeenInit = true;

                // re add this map loading component to the list
               // this.game.Components.Add(this);
            }

            base.Initialize();
        }


        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {
            // TODO: Add your update code here
            switch (currentStep)
            {
                case LoadSteps.LoadFile:
                    this.game.console.Print(Resources.LoadingFile + " " + System.IO.Path.GetFileName(this.mapFile));

                    try
                    {
                        doc.Load(this.mapFile);
                        if (doc.DocumentElement.Name != Resources.XML_map) throw new System.IO.InvalidDataException(Resources.FileIsNotAValidMapFile);
                    }
                    catch (Exception)
                    {
                        throw;
                    }

                    // set the next step
                    currentStep = LoadSteps.ParseTitle;
                    break;

                case LoadSteps.ParseTitle:
                    ParseStringNode(doc.DocumentElement, Resources.XML_title, out this.map.mapTitle);

                    this.game.console.Print(Resources.MapTitle + this.map.mapTitle);
                    currentStep = LoadSteps.ParseBackgroundColor;
                    break;

                case LoadSteps.ParseBackgroundColor:
                    try
                    {
                        ParseColorNode(doc.DocumentElement, Resources.XML_backgroundcolor, out this.map.backgroundColor, true);
                    }
                    catch (Exception)
                    {
                        // if no color element found default will be used
                    }
                    this.game.console.Print(Resources.BackgroundColor + this.map.BackgroundColor.ToString());
                    currentStep = LoadSteps.ParseMapSize;
                    break;

                case LoadSteps.ParseMapSize:
                    ParseIntNode(doc.DocumentElement, Resources.XML_mapwidth, out this.map.mapWidth);
                    ParseIntNode(doc.DocumentElement, Resources.XML_mapheight, out this.map.mapHeight);
                    this.map.Initilize();

                    this.game.console.Print(Resources.MapSize + " " + this.map.mapWidth + ", " + this.map.mapHeight);
                    currentStep = LoadSteps.ParseMapBlockSize;
                    break;

                case LoadSteps.ParseMapBlockSize:
                    ParseFloatNode(doc.DocumentElement, Resources.XML_blockwidth, out this.map.blockWidth);
                    ParseFloatNode(doc.DocumentElement, Resources.XML_blockheight, out this.map.blockHeight);

                    this.game.console.Print(Resources.BlockSize + " " + this.map.blockWidth + ", " + this.map.blockHeight);
                    currentStep = LoadSteps.ParseTextures;
                    break;

                case LoadSteps.ParseTextures:
                    this.game.console.Print(Resources.ParsingTextures);
                    System.Xml.XmlNodeList texturelistItems = doc.DocumentElement.GetElementsByTagName(Resources.XML_textures);
                    if (texturelistItems != null && texturelistItems.Count != 0)
                    {
                        ParseForTextureNodes(texturelistItems);
                    }

                    this.game.console.Print(Resources.NumberOfTexturesLoaded.Replace(Resources.RKEY, this.map.Textures.Count.ToString()));
                    currentStep = LoadSteps.ParseBlocks;
                    break;

                case LoadSteps.ParseBlocks:
                    this.game.console.Print(Resources.ParsingBlocks);
                    ParseBlocks();
                    currentStep = LoadSteps.Done;
                    break;

                    // restore all previous game components
                //    Helpers.RestoreComponents(this.game, gameComponentList);

                  //  CreateMapBoundsController();
                    // set grid/cell size
                    //Components.GridRenderer gr;
                    //gr = Helpers.FindComponent<Components.GridRenderer>(this.game.Components);                    
                    //float mw, mh, bw, bh;
                    //mw = this.map.MapWidth;
                    //mh = this.map.MapHeight;
                    //bw = this.map.BlockWidth;
                    //bh = this.map.BlockHeight;

                    //mw = mw * bw;
                    //mh = mh * bh;

                    //gr.SetGridSize(mw, mh);
                    //gr.SetCellSize(bw, bh);
                    //gr.GridStyle = GridRenderer.GridType.Points;
                    //this.game.console.Print(Resources.Done);
                    //break;

                //case LoadSteps.FadeOut:
                //    if (fadeOut)
                //    {
                //        this.fadeValue -= fadeSpeed * gameTime.ElapsedGameTime.Milliseconds;
                //        this.fadeValue = Math.Max(0, fadeValue);
                //        if (this.fadeValue == 0)
                //        {
                //            currentStep = LoadSteps.Done;
                //        }
                //    }
                //    else
                //    {
                //        currentStep = LoadSteps.Done;
                //    }
                //    break;

                case LoadSteps.Done:
                    // set map file name on map
                    this.map.mapFile = this.mapFile;
                    // set new game map
                    this.game.CurrentMap.Dispose();
                    this.game.CurrentMap = this.map;                    

                // remove this compoent
                    this.game.Components.Remove(this);

                    break;
            }

            base.Update(gameTime);
        }

        //private void CreateMapBoundsController()
        //{
        //    // map width / height controler values
        //    Controllers.MapSize mw = new Controllers.MapSize(this.game.map, Zider.Controllers.MapSize.SourceValue.Width);
        //    Controllers.MapSize mh = new Controllers.MapSize(this.game.map, Zider.Controllers.MapSize.SourceValue.Height);

        //    // find rectangle renderer
        //    Components.RectangleRenderer rr;
        //    rr = Helpers.FindComponent<Components.RectangleRenderer>(this.game.Components);

        //    // get map bounds rect
        //    Components.RectangleRenderer.RectangleInfo rect;
        //    rect = rr.NamedRectangles[Resources.MapBounds];

        //    // rectangle size destinations
        //    Controllers.RectangleInfoValue rw;
        //    Controllers.RectangleInfoValue rh;
        //    rw = new Zider.Controllers.RectangleInfoValue(rect, Zider.Controllers.RectangleInfoValue.SourceValue.Width);
        //    rh = new Zider.Controllers.RectangleInfoValue(rect, Zider.Controllers.RectangleInfoValue.SourceValue.Height);

        //    // create assignment function
        //    Axiom.Controllers.Canned.AssignmentControllerFunction ass;
        //    ass = new Axiom.Controllers.Canned.AssignmentControllerFunction();

        //    // create sync controllers
        //    this.game.ControllerManager.CreateController(mw, rw, ass);
        //    this.game.ControllerManager.CreateController(mh, rh, ass);
        //}

        private void ParseForTextureNodes(System.Xml.XmlNodeList texturelistItems)
        {
            foreach (System.Xml.XmlNode texturesnode in texturelistItems)
            {
                System.Xml.XmlNodeList texturenodes = texturesnode.SelectNodes(Resources.XML_texture);
                foreach (System.Xml.XmlNode tnode in texturenodes)
                {
                    string tName = string.Empty;
                    string fName = string.Empty;
                    ParseStringNode(tnode, Resources.XML_name, out tName);// try to get name
                    ParseStringNode(tnode, Resources.XML_file, out fName);// try to get filename
                    if (tName != string.Empty && fName != string.Empty)
                    {
                        Collections.TextureInfo t;
                        //fName = System.IO.Path.GetFullPath(System.IO.Path.Combine(this.path, fName));
                        t = new Zider.Collections.TextureInfo(this.game, fName, tName);
                        t.Update(null);

                        this.map.Textures.Add(tName, t);
                    }
                }
            }
        }

        private void ParseBlocks()
        {
            System.Xml.XmlNodeList listItems = doc.DocumentElement.GetElementsByTagName(Resources.XML_blocks);
            if (listItems != null)
            {
                foreach (System.Xml.XmlNode blocknode in listItems)
                {
                    System.Xml.XmlNodeList bnodes = blocknode.SelectNodes(Resources.XML_block);
                    foreach (System.Xml.XmlNode bn in bnodes)
                    {
                        ParseBlockInfo(bn);
                    }
                }
            }
        }

        private void ParseBlockInfo(System.Xml.XmlNode blocknode)
        {
            Block block;
            int x, y;
            string tex = string.Empty;
            string type = string.Empty;
            Rectangle sourceRect;
            BlockType bt = BlockType.Platform;

            // get block type
            ParseStringNode(blocknode, Resources.XML_type, out type);
            bt = (BlockType)System.Enum.Parse(bt.GetType(), type, true);

            // get x/y pos for block           
            ParseIntNode(blocknode, Resources.XML_x, out x);
            ParseIntNode(blocknode, Resources.XML_y, out y);

            // get source values
            ParseIntNode(blocknode, Resources.XML_sourceleft, out sourceRect.X);
            ParseIntNode(blocknode, Resources.XML_sourcetop, out sourceRect.Y);
            ParseIntNode(blocknode, Resources.XML_sourcewidth, out sourceRect.Width);
            ParseIntNode(blocknode, Resources.XML_sourceheight, out sourceRect.Height);

            // get texture to use
            ParseStringNode(blocknode, Resources.XML_texture, out tex);

            // check if texture was specified in the texture list
            if (!this.map.Textures.ContainsKey(tex))
            {
                throw new KeyNotFoundException(Resources.TextureNotFoundWithKey.Replace(Resources.RKEY, tex));
            }

            // create block
            block = Helpers.CreateBlock(bt, sourceRect, tex);

            // add block to map
            this.map.SetBlock(x, y, block);
        }

        private void ParseColorNode(System.Xml.XmlNode parentnode, string name, out Color value, bool allowNull)
        {
            System.Xml.XmlNode node = parentnode.SelectSingleNode(name);
            if (node == null && !allowNull) throw new System.IO.InvalidDataException(Resources.CouldNotFindNamedElement.Replace(Resources.RKEY, name));
            System.Xml.XmlAttribute r, g, b, a;
            r = node.Attributes[Resources.XML_r];
            g = node.Attributes[Resources.XML_g];
            b = node.Attributes[Resources.XML_b];
            a = node.Attributes[Resources.XML_a];
            byte rv, gv, bv, av;
            av = 255; // default alpha value

            // check for all three color compoennts
            if (r == null | g == null | b == null)
            {
                throw new MissingFieldException(Resources.ValueIsNotColor + " " + Resources.NamedElement.Replace(Resources.RKEY, name));
            }


            // try to convert values
            if (!byte.TryParse(r.Value, out rv))
            {
                throw new System.IO.InvalidDataException(Resources.ValueIsNotColor + " " + Resources.NamedElement.Replace(Resources.RKEY, name));
            }
            if (!byte.TryParse(g.Value, out gv))
            {
                throw new System.IO.InvalidDataException(Resources.ValueIsNotColor + " " + Resources.NamedElement.Replace(Resources.RKEY, name));
            }
            if (!byte.TryParse(b.Value, out bv))
            {
                throw new System.IO.InvalidDataException(Resources.ValueIsNotColor + " " + Resources.NamedElement.Replace(Resources.RKEY, name));
            }
            if (a != null && !byte.TryParse(a.Value, out av))
            {
                throw new System.IO.InvalidDataException(Resources.ValueIsNotColor + " " + Resources.NamedElement.Replace(Resources.RKEY, name));
            }

            // create color
            Color c = new Color(rv, gv, bv, av);
            value = c;
            return;
        }

        private void ParseIntNode(System.Xml.XmlNode parentnode, string name, out int value)
        {
            System.Xml.XmlNode node = parentnode.SelectSingleNode(name);
            if (node == null) throw new System.IO.InvalidDataException(Resources.CouldNotFindNamedElement.Replace(Resources.RKEY, name));
            if (!int.TryParse(node.InnerText, out value))
            {
                throw new System.IO.InvalidDataException(Resources.ValueIsNotInteger + " " + Resources.NamedElement.Replace(Resources.RKEY, name));
            }
            return;
        }

        private void ParseFloatNode(System.Xml.XmlNode parentnode, string name, out float value)
        {
            System.Xml.XmlNode node = parentnode.SelectSingleNode(name);
            if (node == null) throw new System.IO.InvalidDataException(Resources.CouldNotFindNamedElement.Replace(Resources.RKEY, name));
            if (!float.TryParse(node.InnerText, out value))
            {
                throw new System.IO.InvalidDataException(Resources.ValueIsNotFloat + " " + Resources.NamedElement.Replace(Resources.RKEY, name));
            }
            return;
        }

        private void ParseStringNode(System.Xml.XmlNode parentnode, string name, out string value)
        {
            System.Xml.XmlNode node = parentnode.SelectSingleNode(name);
            if (node == null) throw new System.IO.InvalidDataException(Resources.CouldNotFindNamedElement.Replace(Resources.RKEY, name));
            value = node.InnerText;
            return;
        }

        ///// <summary>
        ///// Allows the game component to draw itself.
        ///// </summary>
        ///// <param name="gameTime">Provides a snapshot of timing values.</param>
        //public override void Draw(GameTime gameTime)
        //{
        //    // TODO: Add your draw code here
        //    int y = this.GraphicsDevice.Viewport.Height; //-game.font.Height; ;
        //    Color col = new Color(fontColor.R, fontColor.G, fontColor.B, (byte)(fadeValue * byte.MaxValue));
        //    for (int idx = info.Count - 1; idx >= 0; idx--)
        //    {
        //        Rectangle sr;
        //        string txt;
        //        txt = info[idx];
        //        sr = game.font.MeasureString(txt);
        //        y -= sr.Height + this.lineSpacing;
        //        game.font.DrawString(new Vector2(0, y), txt, Nuclex.Fonts.TextAlignment.Left | Nuclex.Fonts.TextAlignment.Top, col);
        //    }

        //    base.Draw(gameTime);
        //}
    }
}


