/*
Created by: X
http://www.createdbyx.com/

The Zider engine is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/
*/

#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
#endregion

namespace Zider.Components
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public partial class RectangleRenderer : Microsoft.Xna.Framework.DrawableGameComponent
    {
        public class RectangleInfo
        {
            public Rectangle rect;
            public Color color;
            private bool visible;

            public bool Visible
            {
                get { return visible; }
                set { visible = value; }
            }
	

            public RectangleInfo()
            {
            }

            public RectangleInfo(Color c)
            {
                this.color = c;
                this.rect = Rectangle.Empty;
            }

            public RectangleInfo(Rectangle r)
            {
                this.rect = r;
                this.color = Color.White;
            }

            public RectangleInfo(Rectangle r, Color c)
            {
                this.rect = r;
                this.color = c;
            }

            public RectangleInfo(bool vis)
            {
                this.visible = vis;
            }

            public RectangleInfo(Color c, bool vis)
            {
                this.color = c;
                this.rect = Rectangle.Empty;
                this.visible = vis;
            }

            public RectangleInfo(Rectangle r, bool vis)
            {
                this.rect = r;
                this.color = Color.White;
                this.visible = vis;
            }

            public RectangleInfo(Rectangle r, Color c, bool vis)
            {
                this.rect = r;
                this.color = c;
                this.visible = vis;
            }
        }

        internal protected ZiderGame game;
        private VertexDeclaration decl;
        private VertexBuffer vb;
        private BasicEffect fx;
        private System.Collections.Generic.List<RectangleInfo> rectangles = new List<RectangleInfo>();
        private System.Collections.Generic.Dictionary<string, RectangleInfo> namedRectangles = new Dictionary<string, RectangleInfo>();
        private string camera;
        private Camera rectCamera;


        public string Camera
        {
            get { return camera; }
            set { camera = value; }
        }

        public System.Collections.Generic.List<RectangleInfo> Rectangles
        {
            get { return rectangles; }
        }

        public System.Collections.Generic.Dictionary<string, RectangleInfo> NamedRectangles
        {
            get { return namedRectangles; }
        }
        
        public RectangleRenderer(ZiderGame game)
            : base(game)
        {
            this.game = (ZiderGame)game;
        }

        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            // TODO: Add your initialization code here
            base.Initialize();
            decl = new VertexDeclaration(this.GraphicsDevice, VertexPositionColor.VertexElements);
            fx = new BasicEffect(this.GraphicsDevice, null);

            // generate rectangle
            VertexPositionColor[] verts = new VertexPositionColor[5];
            verts[0] = new VertexPositionColor(Vector3.Zero, Color.White);
            verts[1] = new VertexPositionColor(Vector3.Right, Color.White);
            verts[2] = new VertexPositionColor(Vector3.Right + Vector3.Up, Color.White);
            verts[3] = new VertexPositionColor(Vector3.Up, Color.White);
            verts[4] = new VertexPositionColor(Vector3.Zero, Color.White);
            vb = new VertexBuffer(this.GraphicsDevice, verts.Length * VertexPositionColor.SizeInBytes, ResourceUsage.None, ResourceManagementMode.Automatic);
            vb.SetData<VertexPositionColor>(verts);

            this.rectCamera = this.game.cam;// (Camera)this.game.cam.Clone();
           // this.rectCamera.ProjectionType = Projection.Orthographic;
        }

        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {
            // TODO: Add your update code here
            if (this.rectCamera.NeedsUpdate) this.rectCamera.Update();

            base.Update(gameTime);
        }

        /// <summary>
        /// Allows the game component to draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Draw(GameTime gameTime)
        {
            // TODO: Add your draw code here
            GraphicsDevice g = this.GraphicsDevice;
            g.VertexDeclaration = decl;
            g.Vertices[0].SetSource(vb, 0, VertexPositionColor.SizeInBytes);

            fx.View = this.rectCamera.ViewMatrix;
            fx.Projection = this.rectCamera.ProjectionMatrix;
            // g.Viewport = this.game.cam.Viewport;

            //fx.View = Matrix.CreateLookAt(Vector3.Forward * 100, Vector3.Zero, Vector3.Down);
            //fx.Projection = Matrix.CreateOrthographic(this.GraphicsDevice.Viewport.Width, this.GraphicsDevice.Viewport.Height, 1, 10000);
            // Matrix baseWorld = Matrix.CreateTranslation(-this.GraphicsDevice.Viewport.Width / 2, -this.GraphicsDevice.Viewport.Height / 2, 0);
            Matrix baseWorld = Matrix.CreateTranslation(-this.game.cam.Position);
            fx.VertexColorEnabled = true;
            fx.TextureEnabled = false;

            fx.Begin();
            foreach (EffectPass pass in fx.CurrentTechnique.Passes)
            {
                pass.Begin();
                // draw rectangle list
                foreach (RectangleInfo r in this.rectangles)
                {
                    if (r.Visible)
                    {
                        // setup matrix and color
                        fx.World = Matrix.CreateScale(r.rect.Width, r.rect.Height, 1) *
                                   Matrix.CreateTranslation(r.rect.X, r.rect.Y, 0) * baseWorld;
                        fx.DiffuseColor = r.color.ToVector3();
                        fx.Alpha = r.color.A / byte.MaxValue;

                        // draw rect
                        g.DrawPrimitives(PrimitiveType.LineStrip, 0, 4);
                    }
                }

                // draw named rectangles
                foreach (string key in this.namedRectangles.Keys)
                {
                        RectangleInfo r;
                        r = this.namedRectangles[key];
                                  if (r.Visible)
                    {

                        // setup matrix and color
                        fx.World = Matrix.CreateScale(r.rect.Width, r.rect.Height, 1) *
                                   Matrix.CreateTranslation(r.rect.X, r.rect.Y, 0) * baseWorld;
                        fx.DiffuseColor = r.color.ToVector3();
                        fx.Alpha = r.color.A / byte.MaxValue;

                        // draw rect
                        g.DrawPrimitives(PrimitiveType.LineStrip, 0, 4);
                    }
                }
                
                pass.End();
            }
            fx.End();

            base.Draw(gameTime);
        }
    }
}


