/*
Created by: X
http://www.createdbyx.com/

The Zider engine is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/
*/

#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Graphics;
#endregion

namespace Zider.Components
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public partial class TextureSelector : Microsoft.Xna.Framework.DrawableGameComponent
    {
        public delegate void SelectedTextureEventHandler(string Name, Rectangle sourceRectangle);
        public delegate void CanceledEventHandler();

        private struct BatchItem
        {
            public int X, Y;
            public Color Color;
            public string Text;

            public BatchItem(string txt, int x, int y, Color c)
            {
                this.X = x;
                this.Y = y;
                this.Text = txt;
                this.Color = c;
            }
        }

        private struct TextureItem
        {
            public string Key;
            public int Top;
            public int Height;

            public TextureItem(string k, int top, int h)
            {
                this.Key = k;
                this.Top = top;
                this.Height = h;
            }
        }

        private enum ComponentState
        {
            CalcEntries = 1,
            UserSelection = 2,
            SelectBlock = 3
        }

        private List<TextureItem> textureitems = new List<TextureItem>();
        private List<IGameComponent> gameComponentList;
        protected ZiderGame game;
        protected SpriteBatch sb;
        private MouseState prevstate;
        private int totalHeight;
        private float offset = 0;
        private ComponentState cState = ComponentState.CalcEntries;
        private int selectedIndex = -1;
        private SelectedTextureEventHandler selTex;
        private CanceledEventHandler canceled;

        public event SelectedTextureEventHandler SelectedTexture
        {
            add { this.selTex += value; }
            remove { this.selTex -= value; }
        }

        public event CanceledEventHandler Canceled
        {
            add { this.canceled += value; }
            remove { this.canceled -= value; }
        }

        public TextureSelector(ZiderGame game)
            : base(game)
        {
            this.game = (ZiderGame)game;
        }

        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            base.Initialize();

            // create the sprite batch renderer
            // if (sb != null && sb.IsDisposed == false) 
            sb = new SpriteBatch(this.GraphicsDevice);

        }

        public void Show()
        {
            this.offset = 0;
            this.cState = ComponentState.CalcEntries;
            gameComponentList = Helpers.HoldComponents(this.game, true);
            this.game.Components.Add(this);
        }


        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {
            // TODO: Add your update code here
            MouseState state = Mouse.GetState();
            this.game.PointerPosition = new Vector2(state.X, state.Y);

            // update the offsett
            ZiderGame e = this.game;
            Rectangle r = e.cam.ViewportRectangle;
            // deflate the rect by 5%
            r.Inflate((int)-(r.Width * 0.05f), (int)-(r.Height * 0.05f));
            // check if mouse is NOT in deflated rect
            if (e.pointerPosition.Y < r.Top || e.pointerPosition.Y > r.Bottom)
            {
                // calc cursor vector on screen
                Vector3 v = new Vector3(0, r.Top + (r.Height / 2), 0) - new Vector3(0, e.pointerPosition.Y, 0);
                // normalize vector
                v.Normalize();

                // calc speed to pan
                v *= -0.5f * gameTime.ElapsedGameTime.Milliseconds;

                // pan map by moving camera
                //                e.cam.Move(v);
                offset = Math.Min(this.totalHeight, Math.Max(0, offset + v.Y));
            }


            switch (this.cState)
            {
                case ComponentState.CalcEntries:
                    int y = 0;
                    this.totalHeight = 0;
                    this.textureitems.Clear();
                    int texIndex = 0;

                    // draw textures
                    foreach (string key in this.game.CurrentMap.Textures.Keys)
                    {
                        Texture2D tex;
                        tex = this.game.CurrentMap.Textures[key].Texture;

                        TextureItem item;
                        item = new TextureItem(key, y, tex.Height);
                        // add entry to list
                        this.textureitems.Add(item);

                        // update next draw position
                        y += tex.Height;// +4;

                        // update total height
                        texIndex++;
                        if (texIndex != this.game.CurrentMap.Textures.Count) this.totalHeight += tex.Height;// +4;
                    }
                    this.cState = ComponentState.UserSelection;
                    break;


                case ComponentState.UserSelection:
                    // check if user is canceling
                    if (state.RightButton == ButtonState.Released &&
                        prevstate.RightButton == ButtonState.Pressed)
                    {
                        if (this.canceled != null) this.canceled();

                        // clear and restore previous components
                        this.game.Components.Clear();
                        Helpers.RestoreComponents(this.game, this.gameComponentList);
                        return;
                    }

                    // check for button click
                    if (state.LeftButton == ButtonState.Released &&
                        prevstate.LeftButton == ButtonState.Pressed)
                    {

                        // check for texture selection
                        for (int idx = 0; idx <= this.textureitems.Count - 1; idx++)
                        {
                            TextureItem item;
                            item = this.textureitems[idx];

                            // check if pointer is over a texture
                            if (this.game.PointerPosition.X >= 0 && this.game.PointerPosition.Y >= (item.Top - (int)this.offset) &&
                                this.game.PointerPosition.X <= this.GraphicsDevice.Viewport.Width &&
                                this.game.PointerPosition.Y <= (item.Top - (int)this.offset) + item.Height)
                            {
                                this.cState = ComponentState.SelectBlock;
                                selectedIndex = idx;
                                break; // break form for loop
                            }
                        }
                    }

                    break; // break from switch


                case ComponentState.SelectBlock:
                    // check if user is canceling
                    if (state.RightButton == ButtonState.Released &&
                        prevstate.RightButton == ButtonState.Pressed)
                    {
                        //if (this.canceled != null) this.canceled();

                        //// clear and restore previous components
                        //this.game.Components.Clear();
                        //Helpers.RestoreComponents(this.game, this.gameComponentList);
                        this.cState = ComponentState.UserSelection;
                        
                    } 
                    // check for button click
                    else if (state.LeftButton == ButtonState.Released &&
                        prevstate.LeftButton == ButtonState.Pressed)
                    {
                        // raise  select texture event
                        if (this.selTex != null)
                        {
                            TextureItem item;
                            item = this.textureitems[this.selectedIndex];
                            int sx, sy, w, h;
                            w = (int)this.game.CurrentMap.BlockWidth;
                            h = (int)this.game.CurrentMap.BlockHeight;

                            // calc the block position of the mouse
                            sx = ((int)this.game.PointerPosition.X / w) * w;
                            sy = ((int)this.game.PointerPosition.Y / h) * h;

                            this.selTex(item.Key, new Rectangle(sx, sy, w, h));
                        }

                        // clear and restore previous components
                        this.game.Components.Clear();
                        Helpers.RestoreComponents(this.game, this.gameComponentList);

                        return;
                    }
                    break;

            }

            // store state
            this.prevstate = state;

            base.Update(gameTime);
        }

        /// <summary>
        /// Allows the game component to draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Draw(GameTime gameTime)
        {
            switch (this.cState)
            {
                case ComponentState.UserSelection:
                    List<BatchItem> labels = new List<BatchItem>();

                    // draw textures
                    sb.Begin();
                    for (int idx = 0; idx <= this.textureitems.Count - 1; idx++)
                    {
                        TextureItem item;
                        item = this.textureitems[idx];

                        Texture2D tex;
                        tex = this.game.CurrentMap.Textures[item.Key].Texture;

                        // draw texture
                        sb.Draw(tex, new Rectangle(0, item.Top + -(int)this.offset, tex.Width, tex.Height), Color.White);

                        // add label
                      labels.Add(new BatchItem(item.Key, 0, item.Top + -(int)this.offset, Color.White));
                       //   labels.Add(new BatchItem(item.Key, 0, item.Top, Color.White));
                    }
                    sb.End();

                    // draw labels
                    foreach (BatchItem item in labels)
                    {
                        Rectangle tr;
                        tr = this.game.font.MeasureString(item.Text);
                        this.game.font.DrawString(new Vector2(item.X, item.Y + tr.Height), item.Text, item.Color);
                    }
                    break;

                case ComponentState.SelectBlock:
                    DrawSelectedTexture();
                    break;
            }

            base.Draw(gameTime);
        }

        private void DrawSelectedTexture()
        {
            TextureItem item;
            item = this.textureitems[this.selectedIndex];

            Texture2D tex;
            tex = this.game.CurrentMap.Textures[item.Key].Texture;

            // draw selected texture
            sb.Begin();
            sb.Draw(tex, new Rectangle(0, 0, tex.Width, tex.Height), Color.White);
            sb.End();

        }
    }
}


