/*
Created by: X
http://www.createdbyx.com/

The Zider engine is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/
*/
using System.Diagnostics;
//using System.Data;
using System.Collections;
using System.Collections.Generic;
using System;
using System.ComponentModel;
using Zider.Properties;

namespace Zider.Console
{

    public class StandardCommands : xGameConsoleXNA.GenericCommands.BaseCommands
    {

        private ZiderGame game;
        private List<Axiom.Controllers.Controller> mapBoundsControllers = new List<Axiom.Controllers.Controller>();


        public ZiderGame Engine
        {
            get { return game; }
        }


        #region Commands

        [Category("Engine"), Description("Quits the game.")]
        public string DoQuit(string[] args)
        {
            if (mblnIsConnected && (mobjManager != null))
            {
                this.game.Exit();
            }

            return string.Empty;
        }

        [Category("Engine"), Description("Toggles between fullscreen and windowed modes.")]
        public string ToggleFullScreen(string[] args)
        {
            if (mblnIsConnected && (mobjManager != null))
            {
                this.game.graphics.ToggleFullScreen();
            }

            return string.Empty;
        }

        [Category("Engine"), Description("Loads a map from a file.")]
        public string LoadMap(string[] args)
        {
            if (mblnIsConnected && (mobjManager != null))
            {
                // if autoprocessing is not enabled then we need to process the args
                if (mobjManager.AutoProcessArguments == false)
                {
                    args = mobjManager.ProcessArguments(args);
                }

                // get filename
                string file;
                file = string.Join(" ", args);
                file = System.IO.Path.GetFullPath(file);

                // validate given filename
                if (System.IO.Path.GetDirectoryName(file).IndexOfAny(System.IO.Path.GetInvalidPathChars()) != -1)
                {
                    this.Print(Resources.InvalidPathChars);
                    return null;
                }
                if (System.IO.Path.GetFileName(file).IndexOfAny(System.IO.Path.GetInvalidFileNameChars()) != -1)
                {
                    this.Print(Resources.InvalidFileChars);
                    return null;
                }
                if (System.IO.File.Exists(file) == false)
                {
                    this.Print(Resources.FileDoesNotExist);
                    return null;
                }

                // create and add the maploader component
                Components.MapLoader ml = new Components.MapLoader(this.game, file);
                game.Components.Add(ml);
            }

            return string.Empty;
        }

        [Category("Engine"), Description("Saves the curent map to a file.")]
        public string SaveMap(string[] args)
        {
            if (mblnIsConnected && (mobjManager != null))
            {
                // if autoprocessing is not enabled then we need to process the args
                if (mobjManager.AutoProcessArguments == false)
                {
                    args = mobjManager.ProcessArguments(args);
                }

                // get filename
                string file;
                file = string.Join(" ", args);
                file = System.IO.Path.GetFullPath(file);

                // validate given filename
                if (System.IO.Path.GetDirectoryName(file).IndexOfAny(System.IO.Path.GetInvalidPathChars()) != -1)
                {
                    this.Print(Resources.InvalidPathChars);
                    return null;
                }
                if (System.IO.Path.GetFileName(file).IndexOfAny(System.IO.Path.GetInvalidFileNameChars()) != -1)
                {
                    this.Print(Resources.InvalidFileChars);
                    return null;
                }
                if (System.IO.Directory.Exists(System.IO.Path.GetDirectoryName(file)) == false)
                {
                    this.Print(Resources.DirectoryNameDoesNotExist);
                    return null;
                }

                // save the map
                Classes.MapSaver ms = new Zider.Classes.MapSaver(this.game, this.game.CurrentMap);
                ms.Save(file);
                ms = null;
            }

            return string.Empty;
        }

        [Category("Engine"), Description("Enables/Disables editor mode.")]
        public string StartEditor(string[] args)
        {
            if (mblnIsConnected == false) return string.Empty;

            // add the gui input command
            Zider.Collections.CommandList cl = this.game.inputManager.Commands;
            // clear out any prev commnds
            cl.Clear();
            // add guiinput handler for handling gui input.
            cl.AddLast(new Commands.GUIInput(game.inputManager));
            // add the block game input command
            cl.AddLast(new Commands.BlockEditor(game.inputManager));

            // clear desktop of any controls
            this.game.GUI.Clear();
            // load gui components
            this.game.Console.Print(Resources.LoadingDesktopWidgets + Resources.DottedPause);
            // load desktop widgets
            LoadDesktopGUI();
            this.game.Console.Print(Resources.DottedPause + " " + Resources.Done);

            return string.Empty;
        }

        [Category("Engine"), Description("Shows/Hides the map boundary indicator.")]
        public string ShowBounds(string[] args)
        {
            if (mblnIsConnected == false) return string.Empty;

            // find rectangle renderer
            Components.RectangleRenderer rr;
            rr = Helpers.FindComponent<Components.RectangleRenderer>(this.game.Components);

            // get map bounds rect
            Components.RectangleRenderer.RectangleInfo rect;
            rect = rr.NamedRectangles[Resources.MapBoundsKey];

            bool b = rect.Visible;
            if (args != null && args.Length != 0)
            {
                bool.TryParse(args[0], out b);
                rect.Visible = b;
            }
            else
            {
                // toggle the visibility of the map bounds
                rect.Visible = !rect.Visible;
            }

            return b.ToString();
        }

        [Category("Engine"), Description("Sets the grid style.")]
        public string GridStyle(string[] args)
        {
            if (mblnIsConnected == false) return string.Empty;

            // find grid rendering component
            Components.GridRenderer gr;
            gr = Helpers.FindComponent<Components.GridRenderer>(this.game.Components);


            Components.GridRenderer.GridType gs = gr.GridStyle;
            if (args != null && args.Length != 0)
            {
                try
                {
                    gs = (Zider.Components.GridRenderer.GridType)Enum.Parse(gs.GetType(), args[0], true);
                } catch { }
                gr.GridStyle = gs;
            }
            else
            {
                // toggle the grid style
                switch (gr.GridStyle)
                {
                    case Zider.Components.GridRenderer.GridType.Lines:
                        gr.GridStyle = Zider.Components.GridRenderer.GridType.Points;
                        break;

                    case Zider.Components.GridRenderer.GridType.Points:
                        gr.GridStyle = Zider.Components.GridRenderer.GridType.Lines;
                        break;   
                }
            }

            return gs.ToString();
        }

        [Category("Engine"), Description("Shows/Hides the map grid.")]
        public string ShowGrid(string[] args)
        {
            if (mblnIsConnected == false) return string.Empty;
            // find grid rendering component
            Components.GridRenderer gr;
            gr = Helpers.FindComponent<Components.GridRenderer>(this.game.Components);

            // calc map size and block size
            float mw, mh, bw, bh;
            mw = this.game.CurrentMap.MapWidth;
            mh = this.game.CurrentMap.MapHeight;
            bw = this.game.CurrentMap.BlockWidth;
            bh = this.game.CurrentMap.BlockHeight;

            mw = mw * bw;
            mh = mh * bh;

            // setup grid renderer
            gr.SetGridSize(mw, mh);
            gr.SetCellSize(bw, bh);
            // gr.GridStyle = GridRenderer.GridType.Points;

            bool b = gr.Visible;
            if (args != null && args.Length != 0)
            {
                bool.TryParse(args[0], out b);
                gr.Visible = b;
            }
            else
            {
                // toggle the visibility of the grid
                gr.Visible = !gr.Visible;
            }

            return b.ToString();
        }

        #endregion  Commands

        public StandardCommands(ZiderGame game)
            : base()
        {
            this.game = game;
        }

        public StandardCommands(ZiderGame game, xGameConsoleXNA.Console Console)
            : base(Console)
        {
            this.game = game;
        }

        public override void OnConnect(xGameConsoleXNA.CommandManager Manager, xGameConsoleXNA.CommandExtenderManager ExtenderManager)
        {
            base.OnConnect(Manager, ExtenderManager);
            DoAddCommand(Resources.CMD_quit, new xGameConsoleXNA.CommandCallback(this.DoQuit));
            DoAddCommand(Resources.CMD_fullscreen, new xGameConsoleXNA.CommandCallback(this.ToggleFullScreen));
            DoAddCommand(Resources.CMD_loadmap, new xGameConsoleXNA.CommandCallback(this.LoadMap));
            DoAddCommand(Resources.CMD_editor, new xGameConsoleXNA.CommandCallback(this.StartEditor));
            DoAddCommand(Resources.CMD_showbounds, new xGameConsoleXNA.CommandCallback(this.ShowBounds));
            DoAddCommand(Resources.CMD_showgrid, new xGameConsoleXNA.CommandCallback(this.ShowGrid));
            DoAddCommand(Resources.CMD_savemap, new xGameConsoleXNA.CommandCallback(this.SaveMap));
            DoAddCommand(Resources.CMD_gridstyle, new xGameConsoleXNA.CommandCallback(this.GridStyle));
            this.game.MapChanged += new ZiderGame.MapChangeEventHandler(this.MapChanged);
            CreateMapBoundsController();
        }

        private void MapChanged(object sender, Map value, Map oldvalue)
        {
            // delete controllers then recreate them / simple way of doing it
            // NOTE: The map bounds controllers that are setup hold a reference to the map
            //       so if the map changes, perticularly it's size the map bounds rect will
            //       not be properly updated. Hence the removeal and re-addition of the controlers
            this.DeleteMapBoundsControllers();
            this.CreateMapBoundsController();
        }

        public override void OnDisconnect()
        {
            base.OnDisconnect();
            DeleteMapBoundsControllers();
            this.game.MapChanged -= new ZiderGame.MapChangeEventHandler(this.MapChanged);
            this.mapBoundsControllers = null;
        }

        private void DeleteMapBoundsControllers()
        {
            // clean up map bounds controllers
            foreach (Axiom.Controllers.Controller item in this.mapBoundsControllers)
            {
                this.game.ControllerManager.Controllers.Remove(item);
            }
            this.mapBoundsControllers.Clear();
        }


        private void CreateMapBoundsController()
        {
            // map width / height controler values
            Controllers.MapSize mw = new Controllers.MapSize(this.game.map, Zider.Controllers.MapSize.SourceValue.Width);
            Controllers.MapSize mh = new Controllers.MapSize(this.game.map, Zider.Controllers.MapSize.SourceValue.Height);

            // find rectangle renderer
            Components.RectangleRenderer rr;
            rr = Helpers.FindComponent<Components.RectangleRenderer>(this.game.Components);

            // get map bounds rect
            Components.RectangleRenderer.RectangleInfo rect;
            rect = rr.NamedRectangles[Resources.MapBoundsKey];

            // rectangle size destinations
            Controllers.RectangleInfoValue rw;
            Controllers.RectangleInfoValue rh;
            rw = new Zider.Controllers.RectangleInfoValue(rect, Zider.Controllers.RectangleInfoValue.SourceValue.Width);
            rh = new Zider.Controllers.RectangleInfoValue(rect, Zider.Controllers.RectangleInfoValue.SourceValue.Height);

            // create assignment function
            Axiom.Controllers.Canned.AssignmentControllerFunction ass;
            ass = new Axiom.Controllers.Canned.AssignmentControllerFunction();

            // create sync controllers
            this.mapBoundsControllers.Add(this.game.ControllerManager.CreateController(mw, rw, ass));
            this.mapBoundsControllers.Add(this.game.ControllerManager.CreateController(mh, rh, ass));
        }

        private void LoadDesktopGUI()
        {
            // attempt to create widget
            Classes.Widgets.ButtonTexturesWidget w;
            w = new Zider.Classes.Widgets.ButtonTexturesWidget(
                    Helpers.CreateWidget(this.game, "Zider.Resources.ButtonTexturesWidget.xml", "root"));

            // add to desktop
            this.game.gui.Add(Resources.ButtonTextures, w);

            //NOTE: Setting bounds directly at this point will not position the widget at the top right corner
            //      begause the width "(w.Width + 4)" of the widget has yet to be set. So the fallowing line
            //      of code will not work
            //w.Bounds = new Rectangle(cam.Viewport.Width - (w.Width + 4), 4, 128, 3);

            // position widiget at top right corner
            w.Width = 128;
            w.Height = 64;
            w.Left = game.cam.Viewport.Width - (w.Width + 4);
            w.Top = 4;


            //// attempt to create console widget
            //Classes.Widgets.ConsoleWidget cw;
            //cw = Helpers.CreateWidget<Classes.Widgets.ConsoleWidget>(this.game, "Zider.Resources.ConsoleWidget.xml", "root");

            //// add to desktop
            //game.gui.Desktop.Add(Resources.Console, cw);

            //// start off invisible
            //cw.Visible = false;

            //// center on screen
            //cw.Left = (this.game.cam.ViewportWidth / 2) - (cw.Width / 2);
            //cw.Top = (this.game.cam.ViewportHeight / 2) - (cw.Height / 2);
        }
    }
}
