/*
Created by: X
http://www.createdbyx.com/

The Zider engine is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/
*/
using System;
using System.Collections.Generic;
using System.Text;
using System.Reflection;
using Microsoft.Xna.Framework;
using Zider.Properties;
using xFripperyXNA;

namespace Zider.Dialogs
{
    public class SaveDialog : Classes.GUIWidget
    {
        private Classes.GUIWidget leftWidget;
        private Classes.GUIWidget rightWidget;
        private string leftTexture;
        private string rightTexture;
        private Rectangle leftRect;
        private Rectangle rightRect;
        private Components.TextureSelector ts;

        public void SetValue(Controllers.ButtonTexturesWidgetValue.SourceValue source, int value)
    {
        switch (source)
        {
            case Controllers.ButtonTexturesWidgetValue.SourceValue.Left_SourceLeft:
                this.leftRect.X = value;
                break;

            case Controllers.ButtonTexturesWidgetValue.SourceValue.Left_SourceTop:
                this.leftRect.Y = value;
                break;

            case Controllers.ButtonTexturesWidgetValue.SourceValue.Left_SourceWidth:
                this.leftRect.Width = value;
                break;

            case Controllers.ButtonTexturesWidgetValue.SourceValue.Left_SourceHeight:
                this.leftRect.Height = value;
                break;

            case Controllers.ButtonTexturesWidgetValue.SourceValue.Right_SourceLeft:
                this.rightRect.X = value;
                break;

            case Controllers.ButtonTexturesWidgetValue.SourceValue.Right_SourceTop:
                this.rightRect.Y = value;
                break;

            case Controllers.ButtonTexturesWidgetValue.SourceValue.Right_SourceWidth:
                this.rightRect.Width = value;
                break;

            case Controllers.ButtonTexturesWidgetValue.SourceValue.Right_SourceHeight:
                this.rightRect.Height = value;
                break;
        }    
    }

        public Rectangle RightSourceRectangle
        {
            get { return rightRect; }
            set { rightRect = value; }
        }

        public Rectangle LeftSourceRectangle
        {
            get { return leftRect; }
            set { leftRect = value; }
        }

        public string RightTexture
        {
            get { return rightTexture; }
            set { rightTexture = value; }
        }

        public string LeftTexture
        {
            get { return leftTexture; }
            set { leftTexture = value; }
        }


        public Classes.GUIWidget LeftWidget
        {
            get { return leftWidget; }

            set
            {
                if (this.leftWidget != null) { this.leftWidget.Click -= new xFripperyXNA.ClickEventHandler(LeftButton_Click); }
                leftWidget = value;
                if (this.leftWidget != null) { this.leftWidget.Click += new xFripperyXNA.ClickEventHandler(LeftButton_Click); }
            }
        }

        public Classes.GUIWidget RightWidget
        {
            get { return rightWidget; }
            set
            {
                if (this.rightWidget != null) { this.rightWidget.Click -= new xFripperyXNA.ClickEventHandler(RightButton_Click); }
                rightWidget = value;
                if (this.rightWidget != null) { this.rightWidget.Click += new xFripperyXNA.ClickEventHandler(RightButton_Click); }
            }
        }

        public SaveDialog()
            : base()
        {
            /*  
             // this.bounds = new Rectangle(0, 0, 100, 50);

             Assembly assembly = Assembly.GetExecutingAssembly();
             string file = "Zider.Resources.ButtonTexturesWidget.xml";
             //xFrippery.GUIReader r = new xFrippery.GUIReader();
             //r.Load(assembly.GetManifestResourceStream(file));
             List<Classes.GUIWidget> items;
             items = xFrippery.GUIReader.Load<Classes.GUIWidget>(assembly.GetManifestResourceStream(file));

             // try to find a control with the name 'root' and set it's bounds to this widgets bounds
             // so it will be rendered acuractly and properly
             xFrippery.IControl mainctrl = this.Children.FindControl("root", false, true);
             if (mainctrl != null) { this.bounds = mainctrl.Bounds; }

             // add controls to widget
             foreach (xFrippery.IControl child in items)
             {
                 this.Children.Add(child);
                 child.Parent = this;
             }     

             // extract data stored in the control 'Data" property and set control properties from it
             this.ExtractData(true);  */
        }

        public override void Initilize(ZiderGame game)
        {
                       // call base init 
            base.Initilize(game);
 
            // search for left and right texture button controls
            this.LeftWidget = this.children.FindControl<Classes.GUIWidget>("leftbutton", true, true);
            this.RightWidget = this.children.FindControl<Classes.GUIWidget>("rightbutton", true, true);

            this.leftWidget.TextureName = Resources.NoTexture;
            this.rightWidget.TextureName = Resources.NoTexture;

        }

        void RightButton_Click(object sender, MouseButtons b, Vector2 pos)
        {
            if (sender == null) return;
            if (sender is Commands.GUIInput)
            {
                if (ts == null)
                {
                    Commands.GUIInput i = (Commands.GUIInput)sender;
                    ts = new Zider.Components.TextureSelector(i.InputManager.Engine);
                    ts.Canceled += new Zider.Components.TextureSelector.CanceledEventHandler(TextureSelection_Canceled);
                    this.leftWidget.GetTexture = i.InputManager.Engine.GetGameTexture;
                    this.rightWidget.GetTexture = i.InputManager.Engine.GetGameTexture;
                }
                ts.SelectedTexture += new Zider.Components.TextureSelector.SelectedTextureEventHandler(Right_SelectedTexture);
                ts.Show();
            }
        }

        void LeftButton_Click(object sender, MouseButtons b, Vector2 pos)
        {
            if (sender == null) return;
            if (sender is Commands.GUIInput)
            {
                if (ts == null)
                {
                    Commands.GUIInput i = (Commands.GUIInput)sender;
                    ts = new Zider.Components.TextureSelector(i.InputManager.Engine);
                    ts.Canceled += new Zider.Components.TextureSelector.CanceledEventHandler(TextureSelection_Canceled);
                    this.leftWidget.GetTexture = i.InputManager.Engine.GetGameTexture;
                    this.rightWidget.GetTexture = i.InputManager.Engine.GetGameTexture;
                }
                ts.SelectedTexture += new Zider.Components.TextureSelector.SelectedTextureEventHandler(Left_SelectedTexture);
                ts.Show();
            }
        }

        void TextureSelection_Canceled()
        {
            ts.SelectedTexture -= new Zider.Components.TextureSelector.SelectedTextureEventHandler(Left_SelectedTexture);
            ts.SelectedTexture -= new Zider.Components.TextureSelector.SelectedTextureEventHandler(Right_SelectedTexture);
        }

        void Left_SelectedTexture(string Name, Rectangle sourceRectangle)
        {
            this.leftTexture = Name;
            this.leftRect = sourceRectangle;
            ts.SelectedTexture -= new Zider.Components.TextureSelector.SelectedTextureEventHandler(Left_SelectedTexture);
            this.leftWidget.TextureName = Name;
            this.leftWidget.SourceRectangle = sourceRectangle;
        }

        void Right_SelectedTexture(string Name, Rectangle sourceRectangle)
        {
            this.rightTexture = Name;
            this.rightRect = sourceRectangle;
            ts.SelectedTexture -= new Zider.Components.TextureSelector.SelectedTextureEventHandler(Right_SelectedTexture);
            this.rightWidget.TextureName = Name;
            this.rightWidget.SourceRectangle = sourceRectangle;
        }
    }
}
