/*
Created by: X
http://www.createdbyx.com/

The Zider engine is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/
*/

#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using System.Reflection;
using Zider.Properties;
using Nuclex.Fonts;
#endregion

namespace Zider
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class ZiderGame : Microsoft.Xna.Framework.Game
    {
              public delegate void MapChangeEventHandler(object sender, Map value, Map oldvalue);
  
        internal GraphicsDeviceManager graphics;
        internal ContentManager content;
        internal protected Map map;
        internal protected Vector2 pointerPosition;
        internal protected Vector2 pointerHotSpot;
        internal protected Collections.TextureDictionary guiTextures = new Zider.Collections.TextureDictionary();
        internal protected Texture2D pointerTexture;
        internal protected SpriteBatch sb;
        //internal protected XNAFont font;
        internal protected BitmapFont font;
        internal protected string dialogTexture = "Dialog";
        internal protected string fontTexture = "Simple";
        internal protected string pointerName = "StdPointer";
        // internal protected string noTexture = "NoTexture";
        //internal protected string whiteTexture = "White";
        internal protected string gameTitle = string.Empty;
        internal protected Camera cam;
        internal protected ButtonState pointerState = ButtonState.Released;
        internal Collections.CameraDictionary cameras = new Zider.Collections.CameraDictionary();
        internal Axiom.Controllers.ControllerManager controllerManager;
        internal protected Components.GUIRenderer gui;
        internal protected Components.InputManager inputManager;
        internal string fileName = string.Empty;
        internal xGameConsoleXNA.ConsoleComponent console;
        // internal Dictionary<string, string> mapList = new Dictionary<string,string>();
        internal string gameDescription;
       // internal string startMap;
        private event MapChangeEventHandler mapChanged;


        public void RaiseMapChanged(object sender, Map value, Map oldvalue)
        {
            if (this.mapChanged != null) this.mapChanged(sender, value, oldvalue);
        }
      
       public event MapChangeEventHandler MapChanged
        {
            add { this.mapChanged += value; }
            remove { this.mapChanged -= value; }
        }

        //public string StartMap
        //{
        //    get { return startMap; }
        //    set { startMap = value; }
        //}

        public string GameDescription
        {
            get { return gameDescription; }
            set { gameDescription = value; }
        }

        //public Dictionary<string, string> MapList
        //{
        //    get { return mapList; }
        //}    

        public xGameConsoleXNA.ConsoleComponent Console
        {
            get { return console; }
        }

        public Components.GUIRenderer GUI
        {
            get { return this.gui; }
        }

        public Axiom.Controllers.ControllerManager ControllerManager
        {
            get { return controllerManager; }
            // set { controllerManager = value; }
        }

        public Collections.CameraDictionary Cameras
        {
            get { return cameras = new Zider.Collections.CameraDictionary(); }
        }

        public ButtonState PointerState
        {
            get { return pointerState; }
            set { pointerState = value; }
        }

        public string GameTitle
        {
            get { return gameTitle; }
            set { gameTitle = value; }
        }

        public GraphicsDevice GraphicsDevice
        {
            get { return this.graphics.GraphicsDevice; }
        }

        public Map CurrentMap
        {
            get { return map; }
            set
            {

                Map oldmap = this.map;
                // dispose of current map
                this.map = null;
                // assign new map
                this.map = value;
                this.RaiseMapChanged(this, this.map, oldmap);
                oldmap = null;
            }
        }

        public Texture2D PointerTexture
        {
            get { return pointerTexture; }
            set { pointerTexture = value; }
        }

        public string PointerName
        {
            get { return pointerName; }
            set { pointerName = value; }
        }

        public Vector2 PointerHotSpot
        {
            get { return pointerPosition; }
            set { pointerPosition = value; }
        }

        public Vector2 PointerPosition
        {
            get { return pointerPosition; }
            set { pointerPosition = value; }
        }

        public float PointerPositionX
        {
            get { return pointerPosition.X; }
            set { pointerPosition.X = value; }
        }

        public float PointerPositionY
        {
            get { return pointerPosition.Y; }
            set { pointerPosition.Y = value; }
        }

        public ZiderGame()
        {
            graphics = new GraphicsDeviceManager(this);
            content = new ContentManager(Services);
        }


        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // create the controler manager
            this.controllerManager = new Axiom.Controllers.ControllerManager(this);

            Zider.Properties.Settings sett = Zider.Properties.Settings.Default;

            this.pointerHotSpot = new Vector2(sett.DefaultPointerHotspotX, sett.DefaultPointerHotspotY);

            // create and init a new map
            this.map = new Map(this);
            this.map.Initilize();

            this.sb = new SpriteBatch(this.graphics.GraphicsDevice);

            // create default camera
            cam = new Camera();
            cam.Viewport = this.GraphicsDevice.Viewport;
            cam.ProjectionType = Projection.Orthographic;
            cam.Position = Vector3.Backward;

            // create default Font       
            this.font = content.Load<BitmapFont>(@Resources.DefaultFont);

            // add game loading component
            this.fileName = GetArgs();
            this.Components.Add(new Components.GameLoader(this, this.fileName));

            base.Initialize();
        }

        public Nuclex.Fonts.BitmapFont DefaultFont
        {
            get { return this.font; }
        }

        public bool GetGameTexture(string Name, out Texture2D tex)
        {
            if (this.CurrentMap.Textures.ContainsKey(Name))
            {
                tex = this.CurrentMap.Textures[Name].Texture;
                return true;
            }
            tex = null;
            return false;
        }

        public bool GetGUITexture(string Name, out Texture2D tex)
        {
            if (this.guiTextures.ContainsKey(Name))
            {
                tex = this.guiTextures[Name].Texture;
                return true;
            }

            tex = null;
            return false;
        }


        private string GetArgs()
        {
            string[] tmp = Environment.GetCommandLineArgs();
            return System.IO.Path.GetFullPath(string.Join(" ", tmp, 1, tmp.Length - 1));
        }


        /// <summary>
        /// Load your graphics content.  If loadAllContent is true, you should
        /// load content from both ResourceManagementMode pools.  Otherwise, just
        /// load ResourceManagementMode.Manual content.
        /// </summary>
        /// <param name="loadAllContent">Which type of content to load.</param>
        protected override void LoadGraphicsContent(bool loadAllContent)
        {
            if (loadAllContent)
            {
                // TODO: Load any ResourceManagementMode.Automatic content
                Assembly assembly = Assembly.GetExecutingAssembly();

                // load dialog texture
                string file = "Zider.Resources.Dialog.png";
                Texture2D tex = Texture2D.FromFile(this.GraphicsDevice, assembly.GetManifestResourceStream(file));
                this.guiTextures.Add(this.dialogTexture, new Collections.TextureInfo(tex));

                // load standard mouse pointer texture
                Zider.Properties.Settings sett = Zider.Properties.Settings.Default;
                if (sett.StandardMousePointer != null && sett.StandardMousePointer != string.Empty)
                {
                    file = System.IO.Path.Combine(Environment.CurrentDirectory, sett.StandardMousePointer);
                    file = System.IO.Path.GetFullPath(file);
                    tex = Texture2D.FromFile(this.GraphicsDevice, file);
                    this.guiTextures.Add(this.pointerName, new Collections.TextureInfo(tex, file));
                }
                else
                {
                    file = "Zider.Resources.HandPointer.png";
                    tex = Texture2D.FromFile(this.GraphicsDevice, assembly.GetManifestResourceStream(file));
                    this.guiTextures.Add(this.pointerName, new Collections.TextureInfo(tex));
                }
                this.pointerTexture = tex;

                //// load simple font
                //file = "Zider.Resources.Simple.DDS";
                //tex = Texture2D.FromFile(this.GraphicsDevice, assembly.GetManifestResourceStream(file));
                //this.guiTextures.Add(this.fontTexture, new Collections.TextureInfo(tex));
                //// this.font.Texture = tex;

                // load NoTexture texture
                if (sett.DefaultNoTexture != null && sett.DefaultNoTexture != string.Empty)
                {
                    file = System.IO.Path.Combine(Environment.CurrentDirectory, sett.DefaultNoTexture);
                    file = System.IO.Path.GetFullPath(file);
                    tex = Texture2D.FromFile(this.GraphicsDevice, file);
                    this.guiTextures.Add(Resources.NoTexture, new Collections.TextureInfo(tex, file));
                }
                else
                {
                    file = "Zider.Resources.NoTexture.png";
                    tex = Texture2D.FromFile(this.GraphicsDevice, assembly.GetManifestResourceStream(file));
                    this.guiTextures.Add(Resources.NoTexture, new Collections.TextureInfo(tex));
                }

                // add a white gui texture
                Texture2D wt = new Texture2D(this.GraphicsDevice, 4, 4, 1, ResourceUsage.Dynamic, SurfaceFormat.Color, ResourceManagementMode.Manual);
                byte[] pdata = new byte[4 * 4 * 4];
                for (int idx = 0; idx <= pdata.Length - 1; idx++)
                {
                    pdata[idx] = byte.MaxValue;
                }
                wt.SetData<byte>(pdata);
                this.guiTextures.Add(Resources.WhiteTexture, new Collections.TextureInfo(wt));


            }

            // TODO: Load any ResourceManagementMode.Manual content
        }


        /// <summary>
        /// Unload your graphics content.  If unloadAllContent is true, you should
        /// unload content from both ResourceManagementMode pools.  Otherwise, just
        /// unload ResourceManagementMode.Manual content.  Manual content will get
        /// Disposed by the GraphicsDevice during a Reset.
        /// </summary>
        /// <param name="unloadAllContent">Which type of content to unload.</param>
        protected override void UnloadGraphicsContent(bool unloadAllContent)
        {
            if (unloadAllContent == true)
            {
                this.CurrentMap.Dispose();
                this.guiTextures.Clear();
                this.gui.Clear();
                content.Unload();
            }
        }


        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // update controllers
            this.controllerManager.Update(gameTime);

            // check if camera needs updating
            if (cam.NeedsUpdate) cam.Update();
            // if camera viewport has changed apply camera viewport to graphics device
            if (cam.ViewportChanged)
            {
                this.GraphicsDevice.Viewport = cam.Viewport;
                cam.ViewportChanged = false;
            }

            base.Update(gameTime);
        }


        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            graphics.GraphicsDevice.Clear(this.map.BackgroundColor);

            base.Draw(gameTime);

            // Draw Pointer
            sb.Begin();
            sb.Draw(this.pointerTexture, this.pointerPosition - this.pointerHotSpot, Color.White);
            sb.End();

        }
    }
}