/*
Created by: X
http://www.createdbyx.com/

xFripperyXNA is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/
*/
using System;
using Microsoft;
using System.Collections;
using System.Collections.Generic;

namespace xFripperyXNA
{
    public class ControlCollection: List<IControl>
	{
		protected internal IControl mobjParent;
		
		/// <summary>
        /// Returns a list if controls that have there Name property set.
		/// </summary>
		/// <returns>Returns a ControlCollection object containing any controls that may have been found.</returns>
        /// <remarks>Use this method to filter out controls that only have the Name propery set. Setting a control Name is optional.</remarks>
		public ControlCollection FindControlsWithNames()
		{
			ControlCollection Items = new ControlCollection();
			
			foreach (IControl C in this)
			{
				// check if ID was assigned
				if (C.Name != string.Empty)
				{
					// add control to list
					Items.Add(C);
				}
			}
			
			return Items;
		}
		
		public IControl FindControl(string Name, bool SearchChildren, bool ignoreCase)
		{
			foreach (IControl c in this)
			{
				IControl Tmp;
				Tmp = Helpers.FindControl(c, Name, SearchChildren, ignoreCase);
				if (Tmp != null)
				{
					return Tmp;
				}
			}
			return null;
		}

        public T FindControl<T>(string Name, bool SearchChildren, bool ignoreCase) where T : IControl
        {
            foreach (T c in this)
            {
                T Tmp;
                Tmp = Helpers.FindControl<T>(c, Name, SearchChildren, ignoreCase);
                if (Tmp != null)
                {
                    return Tmp;
                }
            }
            return default(T);
        }

	    public void MoveToFront(IControl Ctrl)
		{
			int Index;
			Index = this.IndexOf(Ctrl);
			if (Index == - 1)
			{
				return;
			}
            this.RemoveAt(Index);
            this.Add(Ctrl);
		}
		
		public void MoveToBack(IControl Ctrl)
		{
			int Index;
            Index = this.IndexOf(Ctrl);
			if (Index == - 1)
			{
				return;
			}
            this.RemoveAt(Index);
            this.Insert(0, Ctrl);
		}
		
		public void Add(ControlCollection items)
		{
			foreach (IControl c in items)
			{
				this.Add(c);
			}
		}

        public ControlCollection(Control Parent)
        {
            this.mobjParent = Parent;
        }

        public ControlCollection()
        {

        }
	}
	
}
